# Copyright 2023 Cisco Systems, Inc. and its affiliates

# mypy: disable-error-code="empty-body"
from uuid import UUID

from catalystwan.endpoints import APIEndpoints, delete, get, post, put
from catalystwan.models.policy.localized import (
    LocalizedPolicy,
    LocalizedPolicyDeviceInfo,
    LocalizedPolicyEditResponse,
    LocalizedPolicyInfo,
)
from catalystwan.models.policy.policy import PolicyId, PolicyPreview
from catalystwan.typed_list import DataSequence


class ConfigurationVEdgeTemplatePolicy(APIEndpoints):
    def change_policy_resource_group(self):
        # POST /template/policy/vedge/{resourceGroupName}/{policyId}
        ...

    @post("/template/policy/vedge")
    def create_vedge_template(self, payload: LocalizedPolicy) -> PolicyId:
        ...

    @delete("/template/policy/vedge/{id}")
    def delete_vedge_template(self, id: UUID) -> None:
        ...

    @put("/template/policy/vedge/{id}")
    def edit_vedge_template(self, id: UUID, payload: LocalizedPolicy) -> LocalizedPolicyEditResponse:
        ...

    @get("/template/policy/vedge", "data")
    def generate_policy_template_list(self) -> DataSequence[LocalizedPolicyInfo]:
        ...

    @get("/template/policy/vedge/devices/{id}", "data")
    def get_device_list_by_policy(self, id: UUID) -> DataSequence[LocalizedPolicyDeviceInfo]:
        ...

    @get("/template/policy/vedge/devices", "data")
    def get_vedge_policy_device_list(self) -> DataSequence[LocalizedPolicyDeviceInfo]:
        ...

    @get("/template/policy/vedge/definition/{id}")
    def get_vedge_template(self, id: UUID) -> LocalizedPolicy:
        ...

    @get("/template/policy/assembly/vedge/{id}")
    def preview_by_id(self, id: UUID) -> PolicyPreview:
        ...



# Copyright 2022 Cisco Systems, Inc. and its affiliates

import unittest
from unittest.mock import MagicMock, patch

from catalystwan.endpoints.monitoring.server_info import ServerInfo
from catalystwan.models.monitoring.server_info import ServerInfoResponse


class TestServerInfo(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_server_info_data = {
            "Achitecture": "x86_64",
            "Available processors": 4,
        }  # Note the spelling error

    @patch("catalystwan.session.ManagerSession")
    def test_get_server_info(self, mock_session):
        # Arrange
        # Create a MagicMock for the response that has a dataobj method
        mock_response = MagicMock()
        # Set up the mock response's dataobj method to return a ServerInfoResponse
        mock_response.dataobj.return_value = ServerInfoResponse(**self.mock_server_info_data)

        # Mock the request method of the ManagerSession to return the mock response
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response

        server_info_api = ServerInfo(mock_session_instance)

        # Act
        response = server_info_api.get_server_info()

        # Assert
        self.assertIsInstance(response, ServerInfoResponse)
        self.assertEqual(response.architecture, self.mock_server_info_data["Achitecture"])
        self.assertEqual(
            response.available_processors,
            self.mock_server_info_data["Available processors"],
        )
        self.assertEqual(response.architecture, "x86_64")
        self.assertEqual(response.available_processors, 4)

        # Ensure the request method was called
        mock_session_instance.request.assert_called_once()


if __name__ == "__main__":
    unittest.main()



# Copyright 2023 Cisco Systems, Inc. and its affiliates

import ipaddress
from enum import Enum
from pathlib import Path
from typing import ClassVar, List, Optional

from pydantic import ConfigDict, Field

from catalystwan.api.templates.bool_str import BoolStr
from catalystwan.api.templates.feature_template import FeatureTemplate, FeatureTemplateValidator

DEFAULT_STATIC_NAT64_SOURCE_VPN_ID = 0
DEFAULT_STATIC_NAT_SOURCE_VPN_ID = 0
DEFAULT_STATIC_PORT_FORWARD_SOURCE_PORT = 0
DEFAULT_STATIC_PORT_FORWARD_TRANSLATE_PORT = 0
DEFAULT_STATIC_PORT_FORWARD_SOURCE_VPN = 0
DEFAULT_ENCAPSULATION_WEIGHT = 1
DEFAULT_VRRP_PRIORITY = 100
DEFAULT_VRRP_TIMER = 1000
DEFAULT_IPV6_VRRP_PRIORITY = 100
DEFAULT_IPV6_VRRP_TIMER = 1000


class SecondaryIPv4Address(FeatureTemplateValidator):
    address: Optional[ipaddress.IPv4Interface] = None


class SecondaryIPv6Address(FeatureTemplateValidator):
    address: Optional[ipaddress.IPv6Interface] = None


class Direction(str, Enum):
    IN = "in"
    OUT = "out"


class AccessList(FeatureTemplateValidator):
    direction: Direction
    acl_name: str = Field(json_schema_extra={"vmanage_key": "acl-name"})
    model_config = ConfigDict(populate_by_name=True)


class DhcpHelperV6(FeatureTemplateValidator):
    address: ipaddress.IPv6Address
    vpn: Optional[int] = None


class NatChoice(str, Enum):
    INTERFACE = "Interface"
    POOL = "Pool"
    LOOPBACK = "Loopback"


class StaticNat66(FeatureTemplateValidator):
    source_prefix: ipaddress.IPv6Interface = Field(json_schema_extra={"vmanage_key": "source-prefix"})
    translated_source_prefix: str = Field(json_schema_extra={"vmanage_key": "translated-source-prefix"})
    source_vpn_id: int = Field(DEFAULT_STATIC_NAT64_SOURCE_VPN_ID, json_schema_extra={"vmanage_key": "source-vpn-id"})
    model_config = ConfigDict(populate_by_name=True)


class StaticNatDirection(str, Enum):
    INSIDE = "inside"
    OUTSIDE = "outside"


class Static(FeatureTemplateValidator):
    source_ip: ipaddress.IPv4Address = Field(json_schema_extra={"vmanage_key": "source-ip"})
    translate_ip: ipaddress.IPv4Address = Field(json_schema_extra={"vmanage_key": "translate-ip"})
    static_nat_direction: StaticNatDirection = Field(
        StaticNatDirection.INSIDE, json_schema_extra={"vmanage_key": "static-nat-direction"}
    )
    source_vpn: int = Field(DEFAULT_STATIC_NAT_SOURCE_VPN_ID, json_schema_extra={"vmanage_key": "source-vpn"})
    model_config = ConfigDict(populate_by_name=True)


class Proto(str, Enum):
    TCP = "tcp"
    UDP = "udp"


class StaticPortForward(FeatureTemplateValidator):
    source_ip: ipaddress.IPv4Address = Field(json_schema_extra={"vmanage_key": "source-ip"})
    translate_ip: ipaddress.IPv4Address = Field(json_schema_extra={"vmanage_key": "translate-ip"})
    static_nat_direction: StaticNatDirection = Field(
        StaticNatDirection.INSIDE, json_schema_extra={"vmanage_key": "static-nat-direction"}
    )
    source_port: int = Field(DEFAULT_STATIC_PORT_FORWARD_SOURCE_PORT, json_schema_extra={"vmanage_key": "source-port"})
    translate_port: int = Field(
        DEFAULT_STATIC_PORT_FORWARD_TRANSLATE_PORT, json_schema_extra={"vmanage_key": "translate-port"}
    )
    proto: Proto
    source_vpn: int = Field(DEFAULT_STATIC_PORT_FORWARD_SOURCE_VPN, json_schema_extra={"vmanage_key": "source-vpn"})
    model_config = ConfigDict(populate_by_name=True)


class CoreRegion(str, Enum):
    CORE = "core"
    CORE_SHARED = "core-shared"


class SecondaryRegion(str, Enum):
    OFF = "off"
    SECONDARY_ONLY = "secondary-only"
    SECONDARY_SHARED = "secondary-shared"


class Encap(str, Enum):
    GRE = "gre"
    IPSEC = "ipsec"


class Encapsulation(FeatureTemplateValidator):
    encap: Encap
    preference: Optional[int] = None
    weight: int = DEFAULT_ENCAPSULATION_WEIGHT


class Mode(str, Enum):
    HUB = "hub"
    SPOKE = "spoke"


class Value(str, Enum):
    DEFAULT = "default"
    MPLS = "mpls"
    METRO_ETHERNET = "metro-ethernet"
    BIZ_INTERNET = "biz-internet"
    PUBLIC_INTERNET = "public-internet"
    LTE = "lte"
    THREEG = "3g"
    RED = "red"
    GREEN = "green"
    BLUE = "blue"
    GOLD = "gold"
    SILVER = "silver"
    BRONZE = "bronze"
    CUSTOM1 = "custom1"
    CUSTOM2 = "custom2"
    CUSTOM3 = "custom3"
    PRIVATE1 = "private1"
    PRIVATE2 = "private2"
    PRIVATE3 = "private3"
    PRIVATE4 = "private4"
    PRIVATE5 = "private5"
    PRIVATE6 = "private6"


class Carrier(str, Enum):
    DEFAULT = "default"
    CARRIER1 = "carrier1"
    CARRIER2 = "carrier2"
    CARRIER3 = "carrier3"
    CARRIER4 = "carrier4"
    CARRIER5 = "carrier5"
    CARRIER6 = "carrier6"
    CARRIER7 = "carrier7"
    CARRIER8 = "carrier8"


class MediaType(str, Enum):
    AUTO_SELECT = "auto-select"
    RJ45 = "rj45"
    SFP = "sfp"


class Speed(str, Enum):
    TEN = "10"
    HUNDRED = "100"
    THOUSAND = "1000"
    TWOANDAHALFTHOUSAND = "2500"
    TENTHOUSAND = "10000"


class Duplex(str, Enum):
    FULL = "full"
    HALF = "half"
    AUTO = "auto"


class Ip(FeatureTemplateValidator):
    addr: ipaddress.IPv4Address
    mac: str


class Ipv4Secondary(FeatureTemplateValidator):
    address: ipaddress.IPv4Address


class TrackAction(str, Enum):
    DECREMENT = "Decrement"
    SHUTDOWN = "Shutdown"


class TrackingObject(FeatureTemplateValidator):
    name: int
    track_action: TrackAction = Field(TrackAction.DECREMENT, json_schema_extra={"vmanage_key": "track-action"})
    decrement: int
    model_config = ConfigDict(populate_by_name=True)


class Vrrp(FeatureTemplateValidator):
    grp_id: int = Field(json_schema_extra={"vmanage_key": "grp-id"})
    priority: int = DEFAULT_VRRP_PRIORITY
    timer: int = DEFAULT_VRRP_TIMER
    track_omp: BoolStr = Field(default=False, json_schema_extra={"vmanage_key": "track-omp"})
    track_prefix_list: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "track-prefix-list"})
    address: Optional[ipaddress.IPv4Address] = Field(
        default=None, json_schema_extra={"data_path": ["ipv4"], "vmanage_key": "address"}
    )
    ipv4_secondary: Optional[List[Ipv4Secondary]] = Field(
        default=None, json_schema_extra={"vmanage_key": "ipv4-secondary"}
    )
    tloc_change_pref: BoolStr = Field(default=False, json_schema_extra={"vmanage_key": "tloc-change-pref"})
    value: int
    tracking_object: Optional[List[TrackingObject]] = Field(
        default=None, json_schema_extra={"vmanage_key": "tracking-object"}
    )
    model_config = ConfigDict(populate_by_name=True)


class Ipv6(FeatureTemplateValidator):
    ipv6_link_local: ipaddress.IPv6Address = Field(json_schema_extra={"vmanage_key": "ipv6-link-local"})
    prefix: Optional[ipaddress.IPv6Interface] = None
    model_config = ConfigDict(populate_by_name=True)


class Ipv6Vrrp(FeatureTemplateValidator):
    grp_id: int = Field(json_schema_extra={"vmanage_key": "grp-id"})
    priority: int = DEFAULT_IPV6_VRRP_PRIORITY
    timer: int = DEFAULT_IPV6_VRRP_TIMER
    track_omp: BoolStr = Field(default=False, json_schema_extra={"vmanage_key": "track-omp"})
    track_prefix_list: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "track-prefix-list"})
    ipv6: Optional[List[Ipv6]] = None
    model_config = ConfigDict(populate_by_name=True)


class CiscoVpnInterfaceModel(FeatureTemplate):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    if_name: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "if-name"})
    interface_description: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "description"})
    poe: Optional[BoolStr] = None
    ipv4_address: Optional[str] = Field(default=None, json_schema_extra={"data_path": ["ip"], "vmanage_key": "address"})
    secondary_ipv4_address: Optional[List[SecondaryIPv4Address]] = Field(
        default=None, json_schema_extra={"data_path": ["ip"], "vmanage_key": "secondary-address"}
    )
    dhcp_ipv4_client: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["ip"], "vmanage_key": "dhcp-client"}
    )
    dhcp_distance: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "dhcp-distance"})
    ipv6_address: Optional[ipaddress.IPv6Interface] = Field(
        default=None, json_schema_extra={"data_path": ["ipv6"], "vmanage_key": "address"}
    )
    dhcp_ipv6_client: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["ipv6"], "vmanage_key": "dhcp-client"}
    )
    secondary_ipv6_address: Optional[List[SecondaryIPv6Address]] = Field(
        default=None, json_schema_extra={"data_path": ["ipv6"], "vmanage_key": "secondary-address"}
    )
    access_list_ipv4: Optional[List[AccessList]] = Field(default=None, json_schema_extra={"vmanage_key": "access-list"})
    dhcp_helper: Optional[List[ipaddress.IPv4Address]] = Field(
        default=None, json_schema_extra={"vmanage_key": "dhcp-helper"}
    )
    dhcp_helper_v6: Optional[List[DhcpHelperV6]] = Field(
        default=None, json_schema_extra={"vmanage_key": "dhcp-helper-v6"}
    )
    tracker: Optional[List[str]] = None
    auto_bandwidth_detect: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "auto-bandwidth-detect"}
    )
    iperf_server: Optional[ipaddress.IPv4Address] = Field(
        default=None, json_schema_extra={"vmanage_key": "iperf-server"}
    )
    nat: Optional[BoolStr] = None
    nat_choice: Optional[NatChoice] = Field(default=None, json_schema_extra={"vmanage_key": "nat-choice"})
    udp_timeout: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "udp-timeout"})
    tcp_timeout: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "tcp-timeout"})
    nat_range_start: Optional[ipaddress.IPv4Address] = Field(
        default=None, json_schema_extra={"vmanage_key": "range-start"}
    )
    nat_range_end: Optional[ipaddress.IPv4Address] = Field(default=None, json_schema_extra={"vmanage_key": "range-end"})
    overload: Optional[BoolStr] = None
    loopback_interface: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "loopback-interface"})
    prefix_length: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "prefix-length"})
    enable: Optional[BoolStr] = None
    nat64: Optional[BoolStr] = None
    nat66: Optional[BoolStr] = None
    static_nat66: Optional[List[StaticNat66]] = Field(default=None, json_schema_extra={"vmanage_key": "static-nat66"})
    static: Optional[List[Static]] = Field(
        default=None, json_schema_extra={"data_path": ["nat"], "vmanage_key": "static"}
    )
    static_port_forward: Optional[List[StaticPortForward]] = Field(
        default=None, json_schema_extra={"vmanage_key": "static-port-forward"}
    )
    enable_core_region: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "enable-core-region"})
    core_region: Optional[CoreRegion] = Field(default=None, json_schema_extra={"vmanage_key": "core-region"})
    secondary_region: Optional[SecondaryRegion] = Field(
        default=None, json_schema_extra={"vmanage_key": "secondary-region"}
    )
    tloc_encapsulation: Optional[List[Encapsulation]] = Field(
        default=None, json_schema_extra={"vmanage_key": "encapsulation", "data_path": ["tunnel-interface"]}
    )
    border: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface"]})
    per_tunnel_qos: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "per-tunnel-qos"})
    per_tunnel_qos_aggregator: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "per-tunnel-qos-aggregator"}
    )
    mode: Optional[Mode] = None
    tunnels_bandwidth: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "tunnels-bandwidth"})
    group: Optional[List[int]] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface"]})
    value: Optional[Value] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "color"]})
    max_control_connections: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "max-control-connections", "data_path": ["tunnel-interface"]}
    )
    control_connections: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "control-connections", "data_path": ["tunnel-interface"]}
    )
    vbond_as_stun_server: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "vbond-as-stun-server", "data_path": ["tunnel-interface"]}
    )
    exclude_controller_group_list: Optional[List[int]] = Field(
        default=None,
        json_schema_extra={"vmanage_key": "exclude-controller-group-list", "data_path": ["tunnel-interface"]},
    )
    vmanage_connection_preference: Optional[int] = Field(
        default=None,
        json_schema_extra={"vmanage_key": "vmanage-connection-preference", "data_path": ["tunnel-interface"]},
    )
    port_hop: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "port-hop", "data_path": ["tunnel-interface"]}
    )
    restrict: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "color"]})
    dst_ip: Optional[ipaddress.IPv4Address] = Field(
        default=None,
        json_schema_extra={"vmanage_key": "dst-ip", "data_path": ["tunnel-interface", "tloc-extension-gre-to"]},
    )
    carrier: Optional[Carrier] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface"]})
    nat_refresh_interval: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "nat-refresh-interval", "data_path": ["tunnel-interface"]}
    )
    hello_interval: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "hello-interval", "data_path": ["tunnel-interface"]}
    )
    hello_tolerance: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "hello-tolerance", "data_path": ["tunnel-interface"]}
    )
    bind: Optional[str] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface"]})
    last_resort_circuit: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "last-resort-circuit", "data_path": ["tunnel-interface"]}
    )
    low_bandwidth_link: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "low-bandwidth-link", "data_path": ["tunnel-interface"]}
    )
    tunnel_tcp_mss_adjust: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "tunnel-tcp-mss-adjust", "data_path": ["tunnel-interface"]}
    )
    clear_dont_fragment: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "clear-dont-fragment", "data_path": ["tunnel-interface"]}
    )
    propagate_sgt: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface"], "vmanage_key": "propagate-sgt"}
    )
    network_broadcast: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "network-broadcast", "data_path": ["tunnel-interface"]}
    )
    all: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]})
    bgp: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]})
    dhcp: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    dns: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]})
    icmp: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    sshd: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    netconf: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    ntp: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]})
    ospf: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    stun: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    snmp: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    https: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["tunnel-interface", "allow-service"]}
    )
    media_type: Optional[MediaType] = Field(default=None, json_schema_extra={"vmanage_key": "media-type"})
    intrf_mtu: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "intrf-mtu"})
    mtu: Optional[int] = None
    tcp_mss_adjust: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "tcp-mss-adjust"})
    tloc_extension: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "tloc-extension"})
    load_interval: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "load-interval"})
    src_ip: Optional[ipaddress.IPv4Address] = Field(
        default=None, json_schema_extra={"vmanage_key": "src-ip", "data_path": ["tloc-extension-gre-from"]}
    )
    xconnect: Optional[str] = Field(default=None, json_schema_extra={"data_path": ["tloc-extension-gre-from"]})
    mac_address: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "mac-address"})
    speed: Optional[Speed] = None
    duplex: Optional[Duplex] = None
    shutdown: Optional[BoolStr] = False
    arp_timeout: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "arp-timeout"})
    autonegotiate: Optional[BoolStr] = None
    ip_directed_broadcast: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "ip-directed-broadcast"}
    )
    icmp_redirect_disable: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "icmp-redirect-disable"}
    )
    qos_adaptive: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "qos-adaptive"})
    period: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["qos-adaptive"]})
    bandwidth_down: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "bandwidth-down", "data_path": ["qos-adaptive", "downstream"]}
    )
    dmin: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["qos-adaptive", "downstream", "range"]})
    dmax: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["qos-adaptive", "downstream", "range"]})
    bandwidth_up: Optional[int] = Field(
        default=None, json_schema_extra={"vmanage_key": "bandwidth-up", "data_path": ["qos-adaptive", "upstream"]}
    )
    umin: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["qos-adaptive", "upstream", "range"]})
    umax: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["qos-adaptive", "upstream", "range"]})
    shaping_rate: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "shaping-rate"})
    qos_map: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "qos-map"})
    qos_map_vpn: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "qos-map-vpn"})
    service_provider: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "service-provider"})
    bandwidth_upstream: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "bandwidth-upstream"})
    bandwidth_downstream: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "bandwidth-downstream"})
    block_non_source_ip: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "block-non-source-ip"}
    )
    rule_name: Optional[str] = Field(
        default=None, json_schema_extra={"vmanage_key": "rule-name", "data_path": ["rewrite-rule"]}
    )
    access_list_ipv6: Optional[List[AccessList]] = Field(
        default=None, json_schema_extra={"data_path": ["ipv6"], "vmanage_key": "access-list"}
    )
    ip: Optional[List[Ip]] = Field(default=None, json_schema_extra={"data_path": ["arp"]})
    vrrp: Optional[List[Vrrp]] = Field(default=None, json_schema_extra={"vmanage_key": "vrrp"})
    ipv6_vrrp: Optional[List[Ipv6Vrrp]] = Field(default=None, json_schema_extra={"vmanage_key": "ipv6-vrrp"})
    enable_sgt_propagation: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["trustsec", "propagate"], "vmanage_key": "sgt"}
    )
    security_group_tag: Optional[int] = Field(
        default=None, json_schema_extra={"data_path": ["trustsec", "static"], "vmanage_key": "sgt"}
    )
    trusted: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["trustsec", "static"]})
    enable_sgt_authorization_and_forwarding: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["trustsec"], "vmanage_key": "enable"}
    )
    enable_sgt_enforcement: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["trustsec", "enforcement"], "vmanage_key": "enable"}
    )
    enforcement_sgt: Optional[int] = Field(
        default=None, json_schema_extra={"data_path": ["trustsec", "enforcement"], "vmanage_key": "sgt"}
    )

    payload_path: ClassVar[Path] = Path(__file__).parent / "DEPRECATED"
    type: ClassVar[str] = "cisco_vpn_interface"



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from socket import getprotobyname
from typing import Dict, List, Optional

from pydantic import BaseModel, RootModel


class ApplicationProtocolEntry(BaseModel):
    name: str


class ApplicationProtocol(BaseModel):
    name: str
    protocol: Optional[str] = None
    port: Optional[str] = None
    entries: Optional[List[ApplicationProtocolEntry]] = None

    def protocol_as_numbers(self) -> List[int]:
        if self.protocol:
            return [getprotobyname(p) for p in self.protocol.split(" ")]
        return []

    def protocol_as_string_of_numbers(self) -> str:
        return " ".join(str(p) for p in self.protocol_as_numbers())


class ApplicationProtocolMap(RootModel):
    root: Dict[str, ApplicationProtocol]



# Copyright 2023 Cisco Systems, Inc. and its affiliates

#  type: ignore
from catalystwan.api.templates.device_variable import DeviceVariable
from catalystwan.api.templates.models.omp_vsmart_model import OMPvSmart
from catalystwan.utils.device_model import DeviceModel

default_omp = OMPvSmart(template_name="omp_1", template_description="default", device_models=[DeviceModel.VEDGE_C8000V])


omp_2 = OMPvSmart(
    template_name="omp_2",
    template_description="some changes",
    device_models=[DeviceModel.VEDGE_C8000V],
    graceful_restart=False,
    send_backup_paths=False,
    shutdown=True,
    holdtime=30,
)

omp_3 = OMPvSmart(
    template_name="omp_3",
    template_description="advanced",
    device_models=[DeviceModel.VEDGE_C8000V],
    graceful_restart=False,
    graceful_restart_timer=DeviceVariable(name="omp_graceful_restart_timer"),
    send_path_limit=DeviceVariable(name="omp_send_path_limit"),
    discard_rejected=DeviceVariable(name="omp_discard_rejected_custom"),
    send_backup_paths=True,
    shutdown=False,
    advertisement_interval=3,
    holdtime=30,
)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

from enum import Enum
from typing import Any, List, Optional, Union

from attr import define, field  # type: ignore

from catalystwan.dataclasses import DataclassBase
from catalystwan.typed_list import DataSequence
from catalystwan.utils.creation_tools import FIELD_NAME, create_dataclass
from catalystwan.utils.personality import Personality
from catalystwan.utils.reachability import Reachability


class DeviceName(Enum):
    VMANAGE: str = "vManage"
    VBOND: str = "vBond"
    VSMART: str = "vSmart"
    EDGE: str = "WAN Edge"
    VEDGE: str = "vEdge"


class StatusName(Enum):
    CONTROL_STATUS: str = "Control Status"
    CONTROL_UP: str = "Control up"
    PARTIAL_CONTROL: str = "Partial"
    CONTROL_DOWN: str = "Control down"


class BfdConnectivityName(Enum):
    FULL: str = "Full WAN Connectivity"
    PARTIAL: str = "Partial WAN Connectivity"
    NO_CONNECTIVITY: str = "No WAN Connectivity"


class InventoryName(Enum):
    TOTAL: str = "Total"
    AUTHORIZED: str = "Authorized"
    DEPLOYED: str = "Deployed"
    STAGING: str = "Staging"


class PercentageDistributionName(Enum):
    BELOW_10_MBPS: str = "less_than_10_mbps"
    BETWEEN_10_AND_100_MBPS: str = "10_mbps_100_mbps"
    BETWEEN_100_AND_500_MBPS: str = "100_mbps_500_mbps"
    ABOVE_500_MBPS: str = "greater_than_500_mbps"


class PercentageDistribution(Enum):
    BELOW_10_MBPS: str = "< 10 Mbps"
    BETWEEN_10_AND_100_MBPS: str = "10 Mbps - 100 Mbps"
    BETWEEN_100_AND_500_MBPS: str = "100 Mbps - 500 Mbps"
    ABOVE_500_MBPS: str = "> 500 Mbps"


class HealthColor(Enum):
    GREEN: str = "green"
    YELLOW: str = "yellow"
    RED: str = "red"


def name_converter(name):
    if name is None:
        return None
    for enum_type in (StatusName, DeviceName, BfdConnectivityName, InventoryName, PercentageDistributionName):
        try:
            return enum_type(name)
        except ValueError:
            pass

    raise ValueError(f"'{name}' is not a valid name.")


@define
class Count(DataclassBase):
    """Dataclass used to handle endpoints that return count of something

    Example:
        Endpoint '/dataservice/clusterManagement/health/summary' returns count of vmanages, which
            contains details_url, name, count, status and status_list
    """

    details_url: str = field(metadata={FIELD_NAME: "detailsURL"})
    count: Optional[int] = field(default=None)
    status: Optional[str] = field(default=None)
    name: Optional[
        Union[DeviceName, StatusName, BfdConnectivityName, InventoryName, PercentageDistributionName]
    ] = field(default=None, converter=name_converter)
    message: Optional[str] = field(default=None)
    status_list: Optional[DataSequence[Count]] = field(default=None, metadata={FIELD_NAME: "statusList"})
    list: Optional[str] = field(default=None)
    value: Optional[int] = field(default=None)
    percentageDistribution: Optional[PercentageDistribution] = field(default=None)

    @value.validator  # type: ignore
    def value_or_count_provided(self, attribute, value):
        """Count dataclass requires count or value attribute in order to show the number of thing it is displaying"""
        if value is None and self.count is None:
            raise ValueError("For 'Count' dataclass 'value' or 'count' attribute must be provided.")


@define
class CertificatesStatus(DataclassBase):
    """Number of certificates with given status"""

    invalid: int
    warning: int
    revoked: Optional[int] = field(default=None)


@define
class ControlStatus(DataclassBase):
    """Number of given control statuses"""

    up: int = field(metadata={FIELD_NAME: "controlUp"})
    partial: int
    down: int = field(metadata={FIELD_NAME: "controlDown"})


@define
class SiteHealth(DataclassBase):
    """Number connectivity to devices on sites with given health"""

    full_connectivity: int = field(metadata={FIELD_NAME: "fullConnectivity"})
    partial_connectivity: int = field(metadata={FIELD_NAME: "partialConnectivity"})
    no_connectivity: int = field(metadata={FIELD_NAME: "noConnectivity"})


@define
class vEdgeHealth(DataclassBase):
    """Number of vEdges with given health"""

    normal: int
    warning: int
    error: int


@define
class vSmartStatus(DataclassBase):
    """Number of vSmarts with given status"""

    up: int
    down: int


@define
class TenantStatus(DataclassBase):
    id: str = field(metadata={FIELD_NAME: "tenantId"})
    name: str = field(metadata={FIELD_NAME: "tenantName"})
    control_status: ControlStatus = field(
        metadata={FIELD_NAME: "controlStatus"}, converter=lambda x: create_dataclass(ControlStatus, x)
    )
    site_health: SiteHealth = field(
        metadata={FIELD_NAME: "siteHealth"}, converter=lambda x: create_dataclass(SiteHealth, x)
    )
    vedge_health: vEdgeHealth = field(
        metadata={FIELD_NAME: "vEdgeHealth"}, converter=lambda x: create_dataclass(vEdgeHealth, x)
    )
    vsmart_status: vSmartStatus = field(
        metadata={FIELD_NAME: "vSmartStatus"}, converter=lambda x: create_dataclass(vSmartStatus, x)
    )


@define
class DeviceHealth(DataclassBase):
    name: str
    personality: Personality = field(converter=Personality)
    uuid: str
    reachability: Reachability = field(converter=Reachability)
    longitude: float
    latitude: float
    health: HealthColor = field(converter=HealthColor)
    qoe: int
    location: str
    site_id: str
    system_ip: str
    device_type: Personality = field(converter=Personality)
    local_system_ip: str
    device_model: str
    software_version: str
    cpu_load: float
    memory_utilization: float
    control_connections: int
    control_connections_up: int
    vsmart_control_connections: int
    expected_vsmart_connections: int
    has_geo_data: bool
    uptime_date: int
    device_groups: List[str]
    connected_vmanages: List[str]
    bfd_sessions_up: int
    bfd_sessions: int
    omp_peers: int
    omp_peers_up: int
    board_serial_number: int
    chassis_number: str
    vpn_ids: Any


@define
class DevicesHealth(DataclassBase):
    total_devices: int
    devices: DataSequence[DeviceHealth]


@define
class LicensedDevices(DataclassBase):
    """Number of total and licensed devices"""

    total_devices: int = field(metadata={FIELD_NAME: "totalDevices"})
    licensed_devices: int = field(metadata={FIELD_NAME: "licensedDevice"})


@define
class DeviceHealthOverview(DataclassBase):
    """Number of devices with given health overview"""

    good: int
    fair: int
    poor: int


@define
class TransportHealth(DataclassBase):
    """Contains information about loss percentage, latency and jitter for a given color."""

    entry_time: int
    jitter: int
    color: str
    loss_percentage: float
    latency: int
    app_probe_class: str
    summary_time: int


@define
class TunnelHealth(DataclassBase):
    """Contains information about health for a given tunnel."""

    name: str
    remote_color: str
    remote_system_ip: str
    local_color: str
    local_system_ip: str
    vqoe_score: float
    jitter: float
    rx_octets: float
    loss_percentage: float
    latency: float
    state: str
    tx_octets: int
    health: HealthColor = field(converter=HealthColor)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from enum import Enum
from pathlib import Path
from typing import ClassVar, List, Optional

from pydantic import ConfigDict, Field

from catalystwan.api.templates.bool_str import BoolStr
from catalystwan.api.templates.feature_template import FeatureTemplate, FeatureTemplateValidator


class Export(FeatureTemplateValidator):
    asn_ip: str = Field(json_schema_extra={"vmanage_key": "asn-ip"})
    model_config = ConfigDict(populate_by_name=True)


class Import(FeatureTemplateValidator):
    asn_ip: str = Field(json_schema_extra={"vmanage_key": "asn-ip"})
    model_config = ConfigDict(populate_by_name=True)


class RouteTargetIpv4(FeatureTemplateValidator):
    vpn_id: int = Field(json_schema_extra={"vmanage_key": "vpn-id"})
    export: List[Export]
    import_: List[Import] = Field(json_schema_extra={"vmanage_key": "import"})
    model_config = ConfigDict(populate_by_name=True)


class RouteTargetIpv6(FeatureTemplateValidator):
    vpn_id: int = Field(json_schema_extra={"vmanage_key": "vpn-id"})
    export: List[Export]
    import_: List[Import] = Field(json_schema_extra={"vmanage_key": "import"})
    model_config = ConfigDict(populate_by_name=True)


class MplsInterface(FeatureTemplateValidator):
    if_name: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "if-name"})
    model_config = ConfigDict(populate_by_name=True)


class AddressFamilyType(str, Enum):
    IPV4_UNICAST = "ipv4-unicast"


class AggregateAddress(FeatureTemplateValidator):
    prefix: str
    as_set: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "as-set"})
    summary_only: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "summary-only"})
    model_config = ConfigDict(populate_by_name=True)


class Ipv6AggregateAddress(FeatureTemplateValidator):
    prefix: str
    as_set: Optional[bool] = Field(False, json_schema_extra={"vmanage_key": "as-set"})
    summary_only: Optional[bool] = Field(False, json_schema_extra={"vmanage_key": "summary-only"})
    model_config = ConfigDict(populate_by_name=True)


class Network(FeatureTemplateValidator):
    prefix: str


class Ipv6Network(FeatureTemplateValidator):
    prefix: str


class Protocol(str, Enum):
    STATIC = "static"
    CONNECTED = "connected"
    OSPF = "ospf"
    OSPFV3 = "ospfv3"
    OMP = "omp"
    EIGRP = "eigrp"
    NAT = "nat"


class Redistribute(FeatureTemplateValidator):
    protocol: Protocol
    route_policy: Optional[str] = Field(default=None, json_schema_extra={"vmanage_key": "route-policy"})
    model_config = ConfigDict(populate_by_name=True)


class AddressFamily(FeatureTemplateValidator):
    family_type: AddressFamilyType = Field(json_schema_extra={"vmanage_key": "family-type"})
    aggregate_address: Optional[List[AggregateAddress]] = Field(
        default=None, json_schema_extra={"vmanage_key": "aggregate-address"}
    )
    ipv6_aggregate_address: Optional[List[Ipv6AggregateAddress]] = Field(
        default=None, json_schema_extra={"vmanage_key": "ipv6-aggregate-address"}
    )
    network: Optional[List[Network]] = None
    ipv6_network: Optional[List[Ipv6Network]] = Field(default=None, json_schema_extra={"vmanage_key": "ipv6-network"})
    paths: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["maximum-paths"]})
    originate: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["default-information"]})
    policy_name: Optional[str] = Field(
        default=None, json_schema_extra={"data_path": ["table-map"], "vmanage_key": "name"}
    )
    filter: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["table-map"]})
    redistribute: Optional[List[Redistribute]] = None
    model_config = ConfigDict(populate_by_name=True)


class NeighborFamilyType(str, Enum):
    IPV4_UNICAST = "ipv4-unicast"
    VPNV4_UNICAST = "vpnv4-unicast"
    VPNV6_UNICAST = "vpnv6-unicast"


class Direction(str, Enum):
    IN = "in"
    OUT = "out"


class RoutePolicy(FeatureTemplateValidator):
    direction: Direction
    pol_name: str = Field(json_schema_extra={"vmanage_key": "pol-name"})
    model_config = ConfigDict(populate_by_name=True)


class NeighborAddressFamily(FeatureTemplateValidator):
    family_type: NeighborFamilyType = Field(json_schema_extra={"vmanage_key": "family-type"})
    prefix_num: Optional[int] = Field(
        default=None, json_schema_extra={"data_path": ["maximum-prefixes"], "vmanage_key": "prefix-num"}
    )
    threshold: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["maximum-prefixes"]})
    restart: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["maximum-prefixes"]})
    warning_only: Optional[bool] = Field(
        default=None, json_schema_extra={"data_path": ["maximum-prefixes"], "vmanage_key": "warning-only"}
    )
    route_policy: Optional[List[RoutePolicy]] = Field(default=None, json_schema_extra={"vmanage_key": "route-policy"})
    model_config = ConfigDict(populate_by_name=True)


class Neighbor(FeatureTemplateValidator):
    address: str
    description: Optional[str] = None
    shutdown: Optional[BoolStr] = None
    remote_as: int = Field(json_schema_extra={"vmanage_key": "remote-as"})
    keepalive: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["timers"]})
    holdtime: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["timers"]})
    if_name: Optional[str] = Field(
        default=None, json_schema_extra={"data_path": ["update-source"], "vmanage_key": "if-name"}
    )
    next_hop_self: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "next-hop-self"})
    send_community: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "send-community"})
    send_ext_community: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "send-ext-community"})
    ebgp_multihop: Optional[int] = Field(default=None, json_schema_extra={"vmanage_key": "ebgp-multihop"})
    password: Optional[str] = None
    send_label: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "send-label"})
    send_label_explicit: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"vmanage_key": "send-label-explicit"}
    )
    as_override: Optional[BoolStr] = Field(default=None, json_schema_extra={"vmanage_key": "as-override"})
    as_number: Optional[int] = Field(
        default=None, json_schema_extra={"data_path": ["allowas-in"], "vmanage_key": "as-number"}
    )
    address_family: Optional[List[NeighborAddressFamily]] = Field(
        default=None, json_schema_extra={"vmanage_key": "address-family"}
    )
    model_config = ConfigDict(populate_by_name=True)


class IPv6NeighborFamilyType(str, Enum):
    IPV6_UNICAST = "ipv6-unicast"


class IPv6NeighborAddressFamily(FeatureTemplateValidator):
    family_type: IPv6NeighborFamilyType = Field(json_schema_extra={"vmanage_key": "family-type"})
    prefix_num: Optional[int] = Field(
        0, json_schema_extra={"data_path": ["maximum-prefixes"], "vmanage_key": "prefix-num"}
    )
    threshold: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["maximum-prefixes"]})
    restart: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["maximum-prefixes"]})
    warning_only: Optional[bool] = Field(
        False, json_schema_extra={"data_path": ["maximum-prefixes"], "vmanage_key": "warning-only"}
    )
    route_policy: Optional[List[RoutePolicy]] = Field(default=None, json_schema_extra={"vmanage_key": "route-policy"})
    model_config = ConfigDict(populate_by_name=True)


class Ipv6Neighbor(FeatureTemplateValidator):
    address: str
    description: Optional[str] = None
    shutdown: Optional[BoolStr] = None
    remote_as: int = Field(default=None, json_schema_extra={"vmanage_key": "remote-as"})
    keepalive: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["timers"]})
    holdtime: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["timers"]})
    if_name: Optional[str] = Field(
        default=None, json_schema_extra={"data_path": ["update-source"], "vmanage_key": "if-name"}
    )
    next_hop_self: Optional[BoolStr] = Field(default=False, json_schema_extra={"vmanage_key": "next-hop-self"})
    send_community: Optional[BoolStr] = Field(default=True, json_schema_extra={"vmanage_key": "send-community"})
    send_ext_community: Optional[BoolStr] = Field(default=True, json_schema_extra={"vmanage_key": "send-ext-community"})
    ebgp_multihop: Optional[int] = Field(1, json_schema_extra={"vmanage_key": "ebgp-multihop"})
    password: Optional[str] = None
    send_label: Optional[BoolStr] = Field(default=False, json_schema_extra={"vmanage_key": "send-label"})
    send_label_explicit: Optional[BoolStr] = Field(
        default=False, json_schema_extra={"vmanage_key": "send-label-explicit"}
    )
    as_override: Optional[BoolStr] = Field(default=False, json_schema_extra={"vmanage_key": "as-override"})
    as_number: Optional[int] = Field(
        default=None, json_schema_extra={"data_path": ["allowas-in"], "vmanage_key": "as-number"}
    )
    address_family: Optional[List[IPv6NeighborAddressFamily]] = Field(
        default=None, json_schema_extra={"vmanage_key": "address-family"}
    )
    model_config = ConfigDict(populate_by_name=True)


class CiscoBGPModel(FeatureTemplate):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    as_num: Optional[str] = Field(default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "as-num"})
    shutdown: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["bgp"]})
    router_id: Optional[str] = Field(default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "router-id"})
    propagate_aspath: Optional[bool] = Field(
        default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "propagate-aspath"}
    )
    propagate_community: Optional[bool] = Field(
        default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "propagate-community"}
    )
    route_target_ipv4: List[RouteTargetIpv4] = Field(
        [], json_schema_extra={"data_path": ["bgp", "target"], "vmanage_key": "route-target-ipv4"}
    )
    route_target_ipv6: List[RouteTargetIpv6] = Field(
        [], json_schema_extra={"data_path": ["bgp", "target"], "vmanage_key": "route-target-ipv6"}
    )
    mpls_interface: Optional[List[MplsInterface]] = Field(
        default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "mpls-interface"}
    )
    external: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["bgp", "distance"]})
    internal: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["bgp", "distance"]})
    local: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["bgp", "distance"]})
    keepalive: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["bgp", "timers"]})
    holdtime: Optional[int] = Field(default=None, json_schema_extra={"data_path": ["bgp", "timers"]})
    always_compare: Optional[bool] = Field(
        default=None, json_schema_extra={"data_path": ["bgp", "best-path", "med"], "vmanage_key": "always-compare"}
    )
    deterministic: Optional[BoolStr] = Field(default=None, json_schema_extra={"data_path": ["bgp", "best-path", "med"]})
    missing_as_worst: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["bgp", "best-path", "med"], "vmanage_key": "missing-as-worst"}
    )
    compare_router_id: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["bgp", "best-path"], "vmanage_key": "compare-router-id"}
    )
    multipath_relax: Optional[BoolStr] = Field(
        default=None, json_schema_extra={"data_path": ["bgp", "best-path", "as-path"], "vmanage_key": "multipath-relax"}
    )
    address_family: Optional[List[AddressFamily]] = Field(
        default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "address-family"}
    )
    neighbor: Optional[List[Neighbor]] = Field(default=None, json_schema_extra={"data_path": ["bgp"]})
    ipv6_neighbor: Optional[List[Ipv6Neighbor]] = Field(
        default=None, json_schema_extra={"data_path": ["bgp"], "vmanage_key": "ipv6-neighbor"}
    )

    payload_path: ClassVar[Path] = Path(__file__).parent / "DEPRECATED"
    type: ClassVar[str] = "cisco_bgp"



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import Optional, Protocol, Type, TypeVar

from packaging.version import Version  # type: ignore

from catalystwan.typed_list import DataSequence
from catalystwan.utils.session_type import SessionType

T = TypeVar("T")


class APIEndpointClientResponse(Protocol):
    """
    Interface to response object. Fits "requests.Response"
    but set of methods is minimal to allow easy migration to another client if needed
    """

    @property
    def text(self) -> str:
        ...

    @property
    def content(self) -> bytes:
        ...

    def dataobj(self, cls: Type[T], sourcekey: Optional[str], validate: bool) -> T:
        ...

    def dataseq(self, cls: Type[T], sourcekey: Optional[str], validate: bool) -> DataSequence[T]:
        ...

    def json(self) -> dict:
        ...


class APIEndpointClient(Protocol):
    """
    Interface to client object.
    We only need a 'request' function and few vmanage session properties obtained from server.
    Matched to fit "requests.Session" but migration to other client is possible.
    At his point not very clean as injection of custom kwargs is possible (and sometimes used)
    """

    def request(self, method: str, url: str, **kwargs) -> APIEndpointClientResponse:
        ...

    @property
    def api_version(self) -> Optional[Version]:
        ...

    @property
    def session_type(self) -> Optional[SessionType]:
        ...

    @property
    def validate_responses(self) -> bool:
        ...



# Copyright 2023 Cisco Systems, Inc. and its affiliates

import unittest
from unittest.mock import patch

from catalystwan.api.basic_api import DeviceStateAPI
from catalystwan.api.packet_capture_api import PacketCaptureAPI
from catalystwan.dataclasses import Device, PacketSetup, Status


class TestPacketCaptureApi(unittest.TestCase):
    def setUp(self):
        self.status = Status("COMPLETED", 1)
        self.status_response = {"fileDownloadStatus": "COMPLETED", "fileSize": 1}
        self.device = Device(
            personality="vedge",
            uuid="mock_uuid",
            id="mock_ip",
            hostname="mock_host",
            reachability="reachable",
            local_system_ip="mock_ip",
            status="normal",
            memUsage=1.0,
            connected_vManages=["192.168.0.1"],
            model="vedge-cloud",
        )
        self.packet_setup = PacketSetup("id", True)

    @patch("catalystwan.session.ManagerSession")
    def test_get_status(self, mock_session):
        # Arrange
        mock_session.get_json.return_value = self.status_response
        # Act
        answer = PacketCaptureAPI(mock_session).get_status(self.packet_setup)
        # Assert
        self.assertEqual(answer, self.status)

    @patch("catalystwan.session.ManagerSession")
    def test_download_capture_session(self, mock_session):
        # Act
        answer = PacketCaptureAPI(mock_session).download_capture_session(self.packet_setup, self.device)
        # Assert
        self.assertEqual(answer, True)

    @patch("catalystwan.session.ManagerSession")
    def test_get_interface_name(self, mock_session):
        # Arrange
        interface_response = {"data": [{"ifname": "GE01"}]}
        mock_session.get_json.return_value = interface_response
        # Act
        answer = PacketCaptureAPI(mock_session).get_interface_name(self.device)
        # Assert
        self.assertEqual(answer, "GE01")

    @patch("catalystwan.session.ManagerSession")
    def test_start_stop(self, mock_session):
        # Arrange
        mock_session.get_json.return_value = None
        packet_capture = PacketCaptureAPI(mock_session)
        packet_capture.packet_channel = self.packet_setup
        # Act
        with packet_capture.start_stop(self.device) as answer:
            pass
        # Assert
        self.assertEqual(answer, None)

    @patch("catalystwan.api.packet_capture_api.time.sleep")
    @patch.object(PacketCaptureAPI, "download_capture_session")
    @patch.object(PacketCaptureAPI, "get_status")
    @patch("catalystwan.session.ManagerSession")
    def test_channel(self, mock_session, mock_get_status, mock_download, mock_sleep):
        # Arrange
        device_capture_response = {"sessionId": "id", "isNewSession": True}
        mock_session.post.return_value.json.return_value = device_capture_response
        mock_get_status.return_value = self.status
        # Act
        with PacketCaptureAPI(mock_session).channel(self.device) as answer:
            pass
        # Assert
        self.assertEqual(answer, self.packet_setup)

    @patch("catalystwan.api.packet_capture_api.time.sleep")
    @patch("catalystwan.session.ManagerSession")
    def test_channel_raise_error(self, mock_session, mock_sleep):
        # Arrange
        device_capture_response = {"sessionId": "id", "isNewSession": False}
        mock_session.post.return_value.json.return_value = device_capture_response
        # Act
        with self.assertRaises(PermissionError) as error:
            with PacketCaptureAPI(mock_session).channel(self.device):
                pass
        # Assert
        self.assertEqual(error.expected, PermissionError)

    @patch.object(PacketCaptureAPI, "start_stop")
    @patch.object(PacketCaptureAPI, "channel")
    @patch.object(DeviceStateAPI, "enable_data_stream")
    @patch("catalystwan.session.ManagerSession")
    def test_get_packets(self, mock_session, mock_enable, mock_channel, mock_start_stop):
        # Arrange
        mock_enable.return_value.__enter__.return_value = None
        mock_channel.return_value.__enter__.return_value = None
        mock_start_stop.return_value.__enter__.return_value = None
        packet_capture = PacketCaptureAPI(mock_session)
        packet_capture.status = self.status
        # Act
        answer = packet_capture.get_packets(self.device, duration_seconds=1)
        # Assert
        self.assertEqual(answer, self.status)

    @patch.object(PacketCaptureAPI, "start_stop")
    @patch.object(PacketCaptureAPI, "channel")
    @patch.object(DeviceStateAPI, "enable_data_stream")
    @patch("catalystwan.session.ManagerSession")
    def test_get_packets_handle_permission_error(self, mock_session, mock_enable, mock_channel, mock_start_stop):
        # Arrange
        mock_enable.return_value.__enter__.return_value = None
        mock_channel.return_value.__enter__.side_effect = PermissionError()
        mock_start_stop.return_value.__enter__.return_value = None
        packet_capture = PacketCaptureAPI(mock_session)
        failed_status = Status("IMPOSSIBLE", None)
        # Act
        answer = packet_capture.get_packets(self.device, duration_seconds=1)
        # Assert
        self.assertEqual(answer, failed_status)



# Copyright 2024 Cisco Systems, Inc. and its affiliates

# This stub provide top-level "public" policy models to be used with PolicyAPI()
from typing import List, Union

from pydantic import Field
from typing_extensions import Annotated

from .centralized import CentralizedPolicy, TrafficDataDirection
from .definitions.access_control_list import AclPolicy
from .definitions.access_control_list_ipv6 import AclIPv6Policy
from .definitions.control import ControlPolicy
from .definitions.device_access import DeviceAccessPolicy
from .definitions.device_access_ipv6 import DeviceAccessIPv6Policy
from .definitions.hub_and_spoke import HubAndSpokePolicy
from .definitions.mesh import MeshPolicy
from .definitions.qos_map import QoSDropType, QoSMapPolicy
from .definitions.rewrite import RewritePolicy
from .definitions.rule_set import RuleSet
from .definitions.security_group import SecurityGroup
from .definitions.traffic_data import TrafficDataPolicy
from .definitions.vpn_membership import VPNMembershipPolicy
from .definitions.zone_based_firewall import ZoneBasedFWPolicy
from .lists import (
    AppList,
    AppProbeClassList,
    ASPathList,
    ClassMapList,
    ColorList,
    CommunityList,
    DataIPv6PrefixList,
    DataPrefixList,
    ExpandedCommunityList,
    FQDNList,
    GeoLocationList,
    IPSSignatureList,
    IPv6PrefixList,
    LocalAppList,
    LocalDomainList,
    MirrorList,
    PolicerList,
    PortList,
    PreferredColorGroupList,
    PrefixList,
    ProtocolNameList,
    RegionList,
    SiteList,
    SLAClassList,
    TLOCList,
    URLAllowList,
    URLBlockList,
    VPNList,
    ZoneList,
)
from .lists_entries import EncapType, PathPreference, PolicerExceedAction
from .localized import LocalizedPolicy
from .policy_definition import (
    Carrier,
    DNSTypeEntryType,
    MultiRegionRole,
    OriginProtocol,
    PathType,
    PLPEntryType,
    PolicyActionType,
    ServiceType,
    TLOCActionType,
)
from .security import SecurityPolicy, UnifiedSecurityPolicy

AnyPolicyDefinition = Annotated[
    Union[
        RuleSet,
        SecurityGroup,
        ZoneBasedFWPolicy,
        TrafficDataPolicy,
        QoSMapPolicy,
        RewritePolicy,
        ControlPolicy,
        VPNMembershipPolicy,
        HubAndSpokePolicy,
        MeshPolicy,
        AclPolicy,
        AclIPv6Policy,
        DeviceAccessPolicy,
        DeviceAccessIPv6Policy,
    ],
    Field(discriminator="type"),
]

AnyPolicyList = Annotated[
    Union[
        AppList,
        AppProbeClassList,
        ASPathList,
        ClassMapList,
        ColorList,
        CommunityList,
        DataIPv6PrefixList,
        DataPrefixList,
        ExpandedCommunityList,
        FQDNList,
        GeoLocationList,
        IPSSignatureList,
        IPv6PrefixList,
        LocalAppList,
        LocalDomainList,
        MirrorList,
        PolicerList,
        PortList,
        PreferredColorGroupList,
        PrefixList,
        ProtocolNameList,
        RegionList,
        SiteList,
        SLAClassList,
        TLOCList,
        URLBlockList,
        URLAllowList,
        VPNList,
        ZoneList,
    ],
    Field(discriminator="type"),
]


__all__ = (
    "AclIPv6Policy",
    "AclPolicy",
    "AnyPolicyList",
    "AnyPolicyList",
    "AppList",
    "AppProbeClassList",
    "ASPathList",
    "Carrier",
    "CentralizedPolicy",
    "ClassMapList",
    "ColorList",
    "CommunityList",
    "ControlPolicy",
    "DataIPv6PrefixList",
    "DataPrefixList",
    "DeviceAccessIPv6Policy",
    "DeviceAccessPolicy",
    "DNSTypeEntryType",
    "EncapType",
    "ExpandedCommunityList",
    "FQDNList",
    "GeoLocationList",
    "HubAndSpokePolicy",
    "IPSSignatureList",
    "IPv6PrefixList",
    "LocalAppList",
    "LocalDomainList",
    "LocalizedPolicy",
    "MeshPolicy",
    "MirrorList",
    "MultiRegionRole",
    "OriginProtocol",
    "PathPreference",
    "PathType",
    "PLPEntryType",
    "PolicerExceedAction",
    "PolicerList",
    "PolicyActionType",
    "PortList",
    "PreferredColorGroupList",
    "PrefixList",
    "ProtocolNameList",
    "QoSDropType",
    "QoSMapPolicy",
    "RegionList",
    "RewritePolicy",
    "RuleSet",
    "SecurityGroup",
    "SecurityPolicy",
    "ServiceType",
    "SiteList",
    "SLAClassList",
    "TLOCActionType",
    "TLOCList",
    "TrafficDataDirection",
    "TrafficDataPolicy",
    "UnifiedSecurityPolicy",
    "URLBlockList",
    "URLAllowList",
    "VPNList",
    "VPNMembershipPolicy",
    "ZoneBasedFWPolicy",
    "ZoneList",
)


def __dir__() -> "List[str]":
    return list(__all__)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

from typing import TYPE_CHECKING

from catalystwan.api.admin_tech_api import AdminTechAPI
from catalystwan.api.administration import (
    AdministrationSettingsAPI,
    ClusterManagementAPI,
    ResourceGroupsAPI,
    SessionsAPI,
    UserGroupsAPI,
    UsersAPI,
)
from catalystwan.api.alarms_api import AlarmsAPI
from catalystwan.api.basic_api import DevicesAPI, DeviceStateAPI
from catalystwan.api.config_device_inventory_api import ConfigurationDeviceInventoryAPI
from catalystwan.api.config_group_api import ConfigGroupAPI
from catalystwan.api.dashboard_api import DashboardAPI
from catalystwan.api.feature_profile_api import SDRoutingFeatureProfilesAPI
from catalystwan.api.logs_api import LogsAPI
from catalystwan.api.omp_api import OmpAPI
from catalystwan.api.packet_capture_api import PacketCaptureAPI
from catalystwan.api.partition_manager_api import PartitionManagerAPI
from catalystwan.api.policy_api import PolicyAPI
from catalystwan.api.resource_pool_api import ResourcePoolAPI
from catalystwan.api.software_action_api import SoftwareActionAPI
from catalystwan.api.speedtest_api import SpeedtestAPI
from catalystwan.api.template_api import TemplatesAPI
from catalystwan.api.tenant_backup_restore_api import TenantBackupRestoreAPI
from catalystwan.api.tenant_management_api import TenantManagementAPI
from catalystwan.api.tenant_migration_api import TenantMigrationAPI
from catalystwan.api.versions_utils import RepositoryAPI
from catalystwan.api.virtual_image_action_api import LxcActionAPI

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession


class APIContainer:
    def __init__(self, session: ManagerSession):
        self.tenant_management = TenantManagementAPI(session)
        self.admin_tech = AdminTechAPI(session)
        self.administration_settings = AdministrationSettingsAPI(session)
        self.alarms = AlarmsAPI(session)
        self.config_device_inventory_api = ConfigurationDeviceInventoryAPI(session)
        self.config_group = ConfigGroupAPI(session)
        self.dashboard = DashboardAPI(session)
        self.devices = DevicesAPI(session)
        self.device_state = DeviceStateAPI(session)
        self.logs = LogsAPI(session)
        self.omp = OmpAPI(session)
        self.packet_capture = PacketCaptureAPI(session)
        self.speedtest = SpeedtestAPI(session)
        self.templates = TemplatesAPI(session)
        self.tenant_backup = TenantBackupRestoreAPI(session)
        self.tenant_migration = TenantMigrationAPI(session)
        self.repository = RepositoryAPI(session)
        self.resource_pool = ResourcePoolAPI(session)
        self.software = SoftwareActionAPI(session)
        self.lxcsoftware = LxcActionAPI(session)
        self.partition = PartitionManagerAPI(session)
        self.users = UsersAPI(session)
        self.cluster_management = ClusterManagementAPI(session)
        self.user_groups = UserGroupsAPI(session)
        self.resource_groups = ResourceGroupsAPI(session)
        self.sessions = SessionsAPI(session)
        self.policy = PolicyAPI(session)
        self.sd_routing_feature_profiles = SDRoutingFeatureProfilesAPI(session)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

# mypy: disable-error-code="empty-body"

from typing import Optional

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.endpoints import APIEndpoints, get


class CloudConnector(BaseModel):
    model_config = ConfigDict(populate_by_name=True)
    cloud_enabled: Optional[bool] = Field(
        default=False, serialization_alias="cloudEnabled", validation_alias="cloudEnabled"
    )
    client_id: Optional[str] = Field(default=None, serialization_alias="clientId", validation_alias="clientId")
    client_secret: Optional[str] = Field(
        default=None, serialization_alias="clientSecret", validation_alias="clientSecret"
    )
    org_name: Optional[str] = Field(default=None, serialization_alias="orgName", validation_alias="orgName")
    affinity: Optional[str] = None
    telemetry_enabled: Optional[bool] = Field(
        default=None, serialization_alias="telemetryEnabled", validation_alias="telemetryEnabled"
    )


class SDAVCCloudConnector(APIEndpoints):
    def disable_cloud_connector(self):
        # PUT /sdavc/cloudconnector
        ...

    def enable_cloud_connector(self):
        # POST /sdavc/cloudconnector
        ...

    @get("/sdavc/cloudconnector")
    def get_cloud_connector(self) -> CloudConnector:
        ...

    def get_cloud_connector_status(self):
        # GET /sdavc/cloudconnector/status
        ...



# Copyright 2022 Cisco Systems, Inc. and its affiliates

import unittest
from unittest.mock import MagicMock, patch

from catalystwan.endpoints.monitoring.security_policy import MonitoringSecurityPolicy
from catalystwan.models.monitoring.security_policy import SecurityPolicyDeviceList


class TestMonitoringSecurityPolicy(unittest.TestCase):
    def setUp(self) -> None:
        self.mock_device_list_data = {
            "amp_down": ["2.2.2.2"],
            "amp_up": ["3.3.3.3"],
            "ips_down": ["2.2.2.2"],
            "ips_up": ["1.1.1.1"],
            "urlf_down": ["3.3.3.3"],
            "urlf_up": ["1.1.1.1", "2.2.2.2"],
            "zbfw_down": [],
            "zbfw_up": ["1.1.1.1", "2.2.2.2", "3.3.3.3"],
        }

    @patch("catalystwan.session.ManagerSession")
    def test_get_security_policy_device_list(self, mock_session):
        # Arrange
        # Create a MagicMock for the response that has a dataobj method
        mock_response = MagicMock()

        # Set up the mock response's dataobj method to return a ServerInfoResponse
        mock_response.dataobj.return_value = SecurityPolicyDeviceList(**self.mock_device_list_data)

        # Mock the request method of the ManagerSession to return the mock response
        mock_session_instance = mock_session.return_value
        mock_session_instance.request.return_value = mock_response

        mon_sec_pol_api = MonitoringSecurityPolicy(mock_session_instance)

        # Act
        response = mon_sec_pol_api.get_device_list()

        # Assert
        self.assertIsInstance(response, SecurityPolicyDeviceList)
        self.assertEqual(response.amp_down, self.mock_device_list_data["amp_down"])
        self.assertEqual(response.amp_down, ["2.2.2.2"])
        self.assertEqual(response.amp_up, self.mock_device_list_data["amp_up"])
        self.assertEqual(response.amp_up, ["3.3.3.3"])
        self.assertEqual(response.ips_down, self.mock_device_list_data["ips_down"])
        self.assertEqual(response.ips_down, ["2.2.2.2"])
        self.assertEqual(response.ips_up, self.mock_device_list_data["ips_up"])
        self.assertEqual(response.ips_up, ["1.1.1.1"])
        self.assertEqual(response.urlf_down, self.mock_device_list_data["urlf_down"])
        self.assertEqual(response.urlf_down, ["3.3.3.3"])
        self.assertEqual(response.urlf_up, self.mock_device_list_data["urlf_up"])
        self.assertEqual(response.urlf_up, ["1.1.1.1", "2.2.2.2"])
        self.assertEqual(response.zbfw_down, self.mock_device_list_data["zbfw_down"])
        self.assertEqual(response.zbfw_down, [])
        self.assertEqual(response.zbfw_up, self.mock_device_list_data["zbfw_up"])
        self.assertEqual(response.zbfw_up, ["1.1.1.1", "2.2.2.2", "3.3.3.3"])

        # Ensure the request method was called
        mock_session_instance.request.assert_called_once()


if __name__ == "__main__":
    unittest.main()



# Copyright 2022 Cisco Systems, Inc. and its affiliates

from enum import Enum


class OperationStatus(Enum):
    IN_PROGRESS = "In progress"
    FAILURE = "Failure"
    SUCCESS = "Success"
    SCHEDULED = "Scheduled"
    SUCCESS_SCHEDULED = "Done - Scheduled"
    VALIDATION_SUCCESS = "Validation success"
    VALIDATION_FAILURE = "Validation failure"


class OperationStatusId(Enum):
    IN_PROGRESS = "in_progress"
    FAILURE = "failure"
    SUCCESS = "success"
    SCHEDULED = "scheduled"
    SUCCESS_SCHEDULED = "success_scheduled"
    VALIDATION_SUCCESS = "validation_success"



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List, Literal
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.models.policy.policy_definition import PolicyDefinitionBase


class Site(BaseModel):
    model_config = ConfigDict(populate_by_name=True)
    site_list: UUID = Field(validation_alias="siteList", serialization_alias="siteList")
    vpn_list: List[UUID] = Field(validation_alias="vpnList", serialization_alias="vpnList")


class VPNMembershipPolicyDefinition(BaseModel):
    model_config = ConfigDict(populate_by_name=True)
    sites: List[Site] = []


class VPNMembershipPolicy(PolicyDefinitionBase):
    model_config = ConfigDict(populate_by_name=True)
    type: Literal["vpnMembershipGroup"] = "vpnMembershipGroup"
    definition: VPNMembershipPolicyDefinition = VPNMembershipPolicyDefinition()

    def add_site(self, site_list: UUID, vpn_lists: List[UUID]) -> Site:
        site = Site(site_list=site_list, vpn_list=vpn_lists)
        self.definition.sites.append(site)
        return site



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, List, Optional, cast

from catalystwan.api.task_status_api import Task
from catalystwan.api.versions_utils import DeviceVersions, RepositoryAPI
from catalystwan.endpoints.configuration_device_actions import (
    InstallActionPayload,
    InstallData,
    InstallDevice,
    InstallInput,
    PartitionActionPayload,
)
from catalystwan.endpoints.configuration_device_inventory import DeviceDetailsResponse
from catalystwan.exceptions import EmptyVersionPayloadError, ImageNotInRepositoryError  # type: ignore
from catalystwan.typed_list import DataSequence
from catalystwan.utils.personality import Personality
from catalystwan.utils.upgrades_helper import get_install_specification, validate_personality_homogeneity
from catalystwan.version import parse_vmanage_version

logger = logging.getLogger(__name__)


if TYPE_CHECKING:
    from catalystwan.session import ManagerSession


class SoftwareActionAPI:
    """
    API methods for software actions. All methods
    are exececutable on all device categories.

    Usage example:
    # Create session
    session = create_manager_session(...)

    # Prepare devices list
    controllers = session.endpoints.configuration_device_inventory.get_device_details('controllers')
    vsmarts = controllers.filter(personality=Personality.VSMART)
    software_image = "viptela-20.7.2-x86_64.tar.gz"

    # Upgrade
    upgrade_id = SoftwareActionAPI(session).install(devices = vsmarts, software_image=software_image)

    # Check upgrade status
    TaskAPI(session, software_action_id).wait_for_completed()
    """

    def __init__(self, session: ManagerSession) -> None:
        self.session = session
        self.repository = RepositoryAPI(self.session)
        self.device_versions = DeviceVersions(self.session)

    def activate(
        self,
        devices: DataSequence[DeviceDetailsResponse],
        version_to_activate: Optional[str] = "",
        image: Optional[str] = "",
    ) -> Task:
        """
        Set chosen version as current version. Requires that selected devices have already version_to_activate
        or image present in their available files.

        Args:
            devices (List[DeviceDetailsResponse]): For those devices software will be activated
            version_to_activate (Optional[str]): version to be set as current version
            image (Optional[str]): software image name in available files

            Notice: Have to pass one of those arguments (version_to_activate, image)

        Raises:
            EmptyVersionPayloadError: If selected version_to_activate or image not detected in available files

        Returns:
            str: Activate software action id
        """
        validate_personality_homogeneity(devices)
        if image and not version_to_activate:
            version = cast(str, self.repository.get_image_version(image))
        elif version_to_activate and not image:
            version = cast(str, version_to_activate)
        else:
            raise ValueError("You can not provide software_image and image version at the same time!")

        if not version:
            raise ImageNotInRepositoryError(
                "Based on provided arguments, software version to activate on device(s) cannot be detected."
            )

        payload_devices = self.device_versions.get_device_available(version, devices)
        for device in payload_devices:
            if not device.version:
                raise EmptyVersionPayloadError("PartitionDevice payload contains entry with empty version field.")

        device_type = get_install_specification(devices.first()).device_type.value
        partition_payload = PartitionActionPayload(
            action="changepartition", devices=[dev for dev in payload_devices], device_type=device_type
        )

        partition_action = self.session.endpoints.configuration_device_actions.process_mark_change_partition(
            payload=partition_payload
        )

        return Task(self.session, partition_action.id)

    def install(
        self,
        devices: DataSequence[DeviceDetailsResponse],
        reboot: bool = False,
        sync: bool = True,
        v_edge_vpn: int = 0,
        v_smart_vpn: int = 0,
        image: Optional[str] = None,
        image_version: Optional[str] = None,
        downgrade_check: bool = True,
        remote_server_name: Optional[str] = None,
        remote_image_filename: Optional[str] = None,
    ) -> Task:
        """
        Method to install new software

        Args:
            devices (List[DeviceDetailsResponse]): For those devices software will be activated
            reboot (bool, optional): reboot device after action end
            sync (bool, optional): Synchronize settings. Defaults to True.
            v_edge_vpn (int, optional): vEdge VPN
            v_smart_vpn (int, optional): vSmart VPN
            image (str): path to software image or its name from available files
            image_version (str): version of software image
            downgrade_check (bool, optional): perform a downgrade check when applicable
            remote_server_name (str): name of configured Remote Server
            remote_image_filename (str): filename to choose from selected Remote Server

            Notice: Have to pass one of those:
                - image_version
                - image
                - remote_server_name and remote_image_filename

        Raises:
            ValueError: Raise error if downgrade in certain cases or wrong arguments combination provided
            ImageNotInRepositoryError: If selected image, image_version or remote_image_filename not found

        Returns:
            Task: Task object representing started install process
        """
        validate_personality_homogeneity(devices)

        if (
            sum(
                [
                    image is not None,
                    image_version is not None,
                    all([remote_server_name is not None, remote_image_filename is not None]),
                ]
            )
            != 1
        ):
            raise ValueError(
                "Please provide one option to detect software to install. "
                "Pick either 'image', 'image_version', or both 'remote_server_name' and 'remote_image_filename'."
            )

        # FIXME downgrade_check will be supported when software images from Remote Server will have versions fields
        if remote_server_name and remote_image_filename and downgrade_check:
            raise ValueError("Downgrade check is not supported for install action for images from Remote Server.")

        version, remote_image_details = None, None
        if image:
            version = cast(str, self.repository.get_image_version(image))
        if image_version:
            version = cast(str, image_version)
        if remote_server_name and remote_image_filename:
            remote_image_details = self.repository.get_remote_image(remote_image_filename, remote_server_name)

        if not any([version, remote_image_details]):
            raise ImageNotInRepositoryError(
                "Based on provided arguments, software version to install on device(s) cannot be detected."
            )

        install_specification = get_install_specification(devices.first(), remote=bool(remote_image_details))
        install_devices = [
            InstallDevice(**device.model_dump(by_alias=True))
            for device in self.device_versions.get_device_list(devices)
        ]

        if version:
            input = InstallInput(
                v_edge_vpn=v_edge_vpn,
                v_smart_vpn=v_smart_vpn,
                family=install_specification.family.value,
                version=version,
                version_type=install_specification.version_type.value,
                reboot=reboot,
                sync=sync,
            )
        else:
            input = InstallInput(
                v_edge_vpn=v_edge_vpn,
                v_smart_vpn=v_smart_vpn,
                data=[
                    InstallData(
                        family=install_specification.family.value,
                        version=remote_image_details.version_id,  # type: ignore
                        remote_server_id=remote_image_details.remote_server_id,  # type: ignore
                        version_id=remote_image_details.version_id,  # type: ignore
                    )
                ],
                version_type=install_specification.version_type.value,
                reboot=reboot,
                sync=sync,
            )

        device_type = install_specification.device_type.value
        install_payload = InstallActionPayload(
            action="install", input=input, devices=install_devices, device_type=device_type
        )

        if downgrade_check and devices.first().personality in (Personality.VMANAGE, Personality.EDGE):
            self._downgrade_check(
                install_payload.devices,
                install_payload.input.version,  # type: ignore
                install_specification.family.value,  # type: ignore
            )

        install_action = self.session.endpoints.configuration_device_actions.process_install_operation(
            payload=install_payload
        )

        return Task(self.session, install_action.id)

    def _downgrade_check(self, payload_devices: List[InstallDevice], version_to_upgrade: str, family: str) -> None:
        """
        Check if upgrade operation is not actually a downgrade opeartion.
        If so, in some cases action is being blocked.

        Args:
            version_to_upgrade (str): version to upgrade
            payload_devices List[InstallDevice]: list of Devices to check downgrade possibility

        Raises:
            ValueError: If for any of the devices upgrade action is denied
        """
        incorrect_devices = []
        devices_versions_repo = self.repository.get_devices_versions_repository()
        for device in payload_devices:
            current_version = parse_vmanage_version(devices_versions_repo[device.device_id].current_version)
            upgrade_version = parse_vmanage_version(version_to_upgrade)
            # check if downgrade
            if current_version > upgrade_version:
                logger.warning(
                    f"Requested to downgrade device: {device.device_id} from {current_version} to {upgrade_version}"
                )
                # allow vmanage downgrade only if major and minor version match, downgrade for other devices is allowed
                if family == "vmanage":
                    if not current_version.release[:2] == upgrade_version.release[:2]:
                        logger.error(
                            f"Blocking downgrade of device: {device.device_id} "
                            f"from {current_version.release} to {upgrade_version.release}"
                        )
                        incorrect_devices.append(device)

        if incorrect_devices:
            raise ValueError(
                f"Current version of devices with id's {incorrect_devices} is "
                "higher than upgrade version. Action denied!"
            )



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from catalystwan.api.templates.models.cisco_aaa_model import CiscoAAAModel
from catalystwan.api.templates.models.cisco_banner_model import CiscoBannerModel
from catalystwan.api.templates.models.cisco_bfd_model import CiscoBFDModel
from catalystwan.api.templates.models.cisco_logging_model import CiscoLoggingModel
from catalystwan.api.templates.models.cisco_ntp_model import CiscoNTPModel
from catalystwan.api.templates.models.cisco_omp_model import CiscoOMPModel
from catalystwan.api.templates.models.cisco_ospf import CiscoOSPFModel
from catalystwan.api.templates.models.cisco_secure_internet_gateway import CiscoSecureInternetGatewayModel
from catalystwan.api.templates.models.cisco_snmp_model import CiscoSNMPModel
from catalystwan.api.templates.models.cisco_system import CiscoSystemModel
from catalystwan.api.templates.models.cisco_vpn_interface_model import CiscoVpnInterfaceModel
from catalystwan.api.templates.models.cisco_vpn_model import CiscoVPNModel
from catalystwan.api.templates.models.omp_vsmart_model import OMPvSmart
from catalystwan.api.templates.models.security_vsmart_model import SecurityvSmart
from catalystwan.api.templates.models.system_vsmart_model import SystemVsmart

available_models = {
    "cisco_aaa": CiscoAAAModel,
    "cisco_bfd": CiscoBFDModel,
    "cisco_banner": CiscoBannerModel,
    "cisco_ntp": CiscoNTPModel,
    "cisco_ospf": CiscoOSPFModel,
    "cisco_logging": CiscoLoggingModel,
    "omp_vsmart": OMPvSmart,
    "security_vsmart": SecurityvSmart,
    "system_vsmart": SystemVsmart,
    "cisco_vpn_interface": CiscoVpnInterfaceModel,
    "cisco_system": CiscoSystemModel,
    "cisco_vpn": CiscoVPNModel,
    "cisco_snmp": CiscoSNMPModel,
    "cisco_system": CiscoSystemModel,
    "cisco_secure_internet_gateway": CiscoSecureInternetGatewayModel,
    "cisco_omp": CiscoOMPModel,
}



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from typing import List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field, IPvAnyAddress, RootModel, field_validator
from typing_extensions import Annotated

from catalystwan.models.policy.policy import (
    AdvancedMalwareProtectionAssemblyItem,
    DNSSecurityAssemblyItem,
    IntrusionPreventionAssemblyItem,
    NGFirewallAssemblyItem,
    PolicyCreationPayload,
    PolicyDefinition,
    PolicyInfo,
    SSLDecryptionAssemblyItem,
    URLFilteringAssemblyItem,
    ZoneBasedFWAssemblyItem,
)

SecurityPolicyAssemblyItem = Annotated[
    Union[
        ZoneBasedFWAssemblyItem,
        IntrusionPreventionAssemblyItem,
        URLFilteringAssemblyItem,
        AdvancedMalwareProtectionAssemblyItem,
        DNSSecurityAssemblyItem,
        SSLDecryptionAssemblyItem,
    ],
    Field(discriminator="type"),
]

UnifiedSecurityPolicyAssemblyItem = Annotated[
    Union[
        NGFirewallAssemblyItem,
        DNSSecurityAssemblyItem,
    ],
    Field(discriminator="type"),
]

FailureMode = Literal[
    "open",
    "close",
]

ZoneToNoZoneInternet = Literal[
    "allow",
    "deny",
]


class HighSpeedLoggingEntry(BaseModel):
    vrf: str
    server_ip: IPvAnyAddress = Field(alias="serverIp")
    port: str
    source_interface: Optional[str] = Field(alias="sourceInterface")
    model_config = ConfigDict(populate_by_name=True)


class HighSpeedLoggingList(BaseModel):
    entries: List[HighSpeedLoggingEntry]


class LoggingEntry(BaseModel):
    vpn: str
    server_ip: IPvAnyAddress = Field(alias="serverIP")
    model_config = ConfigDict(populate_by_name=True)


class SecurityPolicySettings(BaseModel):
    logging: Optional[List[LoggingEntry]] = None
    failure_mode: Optional[FailureMode] = Field(default=None, alias="failureMode")
    zone_to_no_zone_internet: ZoneToNoZoneInternet = Field(default="deny", alias="zoneToNozoneInternet")
    tcp_syn_flood_limit: Optional[str] = Field(default=None, alias="tcpSynFloodLimit")
    high_speed_logging: Optional[HighSpeedLoggingEntry] = Field(default=None, alias="highSpeedLogging")
    audit_trail: Optional[str] = Field(default=None, alias="auditTrail")
    platform_match: Optional[str] = Field(default=None, alias="platformMatch")
    model_config = ConfigDict(populate_by_name=True)


class UnifiedSecurityPolicySettings(BaseModel):
    tcp_syn_flood_limit: Optional[str] = Field(default=None, alias="tcpSynFloodLimit")
    max_incomplete_tcp_limit: Optional[str] = Field(default=None, alias="maxIncompleteTcpLimit")
    max_incomplete_udp_limit: Optional[str] = Field(default=None, alias="maxIncompleteUdpLimit")
    max_incomplete_icmp_limit: Optional[str] = Field(default=None, alias="maxIncompleteIcmpLimit")
    high_speed_logging: Optional[HighSpeedLoggingList] = Field(default=None, alias="highSpeedLogging")
    model_config = ConfigDict(populate_by_name=True)


class SecurityPolicyDefinition(PolicyDefinition):
    assembly: List[SecurityPolicyAssemblyItem] = []
    settings: SecurityPolicySettings = SecurityPolicySettings()


class UnifiedSecurityPolicyDefinition(PolicyDefinition):
    assembly: List[UnifiedSecurityPolicyAssemblyItem] = []
    settings: UnifiedSecurityPolicySettings = UnifiedSecurityPolicySettings()


class SecurityPolicy(PolicyCreationPayload):
    policy_mode: Literal["security"] = Field(default="security", alias="policyMode")
    policy_type: str = Field(default="feature", alias="policyType")
    policy_use_case: str = Field(default="custom", alias="policyUseCase")
    policy_definition: SecurityPolicyDefinition = Field(default=SecurityPolicyDefinition(), alias="policyDefinition")

    def add_item(self, item: SecurityPolicyAssemblyItem) -> None:
        self.policy_definition.assembly.append(item)

    def add_zone_based_fw(self, definition_id: UUID) -> None:
        self.add_item(ZoneBasedFWAssemblyItem(definition_id=definition_id))

    def add_dns_security(self, definition_id: UUID) -> None:
        self.add_item(DNSSecurityAssemblyItem(definition_id=definition_id))

    def add_intrusion_prevention(self, definition_id: UUID) -> None:
        self.add_item(IntrusionPreventionAssemblyItem(definition_id=definition_id))

    def add_url_filtering(self, definition_id: UUID) -> None:
        self.add_item(URLFilteringAssemblyItem(definition_id=definition_id))

    def add_advanced_malware_protection(self, definition_id: UUID) -> None:
        self.add_item(AdvancedMalwareProtectionAssemblyItem(definition_id=definition_id))

    def add_ssl_decryption(self, definition_id: UUID) -> None:
        self.add_item(SSLDecryptionAssemblyItem(definition_id=definition_id))

    @field_validator("policy_definition", mode="before")
    @classmethod
    def try_parse(cls, policy_definition):
        if isinstance(policy_definition, str):
            return SecurityPolicyDefinition.model_validate_json(policy_definition)
        return policy_definition


class UnifiedSecurityPolicy(PolicyCreationPayload):
    policy_mode: Literal["unified"] = Field("unified", alias="policyMode")
    policy_type: str = Field("feature", alias="policyType")
    policy_use_case: str = Field("custom", alias="policyUseCase")
    policy_definition: UnifiedSecurityPolicyDefinition = Field(
        default=UnifiedSecurityPolicyDefinition(), alias="policyDefinition"
    )

    def add_item(self, item: UnifiedSecurityPolicyAssemblyItem) -> None:
        self.policy_definition.assembly.append(item)

    def add_ng_firewall(self, definition_id: UUID) -> NGFirewallAssemblyItem:
        ng_fw = NGFirewallAssemblyItem(definition_id=definition_id)
        self.add_item(ng_fw)
        return ng_fw

    def add_dns_security(self, definition_id: UUID) -> None:
        self.add_item(DNSSecurityAssemblyItem(definition_id=definition_id))

    @field_validator("policy_definition", mode="before")
    @classmethod
    def try_parse(cls, policy_definition):
        if isinstance(policy_definition, str):
            return UnifiedSecurityPolicyDefinition.model_validate_json(policy_definition)
        return policy_definition


AnySecurityPolicy = Annotated[Union[SecurityPolicy, UnifiedSecurityPolicy], Field(discriminator="policy_mode")]


class SecurityPolicyRoot(RootModel):
    root: AnySecurityPolicy


class SecurityPolicyEditResponse(BaseModel):
    master_templates_affected: List[str] = Field(default=[], alias="masterTemplatesAffected")


class SecurityPolicyInfo(SecurityPolicy, PolicyInfo):
    virtual_application_templates: List[str] = Field(alias="virtualApplicationTemplates")
    supported_devices: List[str] = Field(alias="supportedDevices")


class UnifiedSecurityPolicyInfo(UnifiedSecurityPolicy, PolicyInfo):
    virtual_application_templates: List[str] = Field(alias="virtualApplicationTemplates")
    supported_devices: List[str] = Field(alias="supportedDevices")


AnySecurityPolicyInfo = Annotated[
    Union[SecurityPolicyInfo, UnifiedSecurityPolicyInfo], Field(discriminator="policy_mode")
]


class SecurityPolicyInfoRoot(RootModel):
    root: AnySecurityPolicyInfo



# Copyright 2023 Cisco Systems, Inc. and its affiliates

import unittest
from unittest.mock import patch
from urllib.error import HTTPError

from catalystwan.api.basic_api import DeviceStateAPI
from catalystwan.api.speedtest_api import SpeedtestAPI
from catalystwan.dataclasses import Device, Speedtest


class TestSpeedTestAPI(unittest.TestCase):
    def setUp(self):
        self.device = Device(
            personality="vedge",
            uuid="mock_uuid",
            id="mock_ip",
            hostname="mock_host",
            reachability="reachable",
            local_system_ip="mock_ip",
            status="normal",
            memUsage=1.0,
            connected_vManages=["192.168.0.1"],
            model="vedge-cloud",
        )
        self.device_unreachable = Device(
            personality="vedge",
            uuid="mock_uuid",
            id="mock_ip",
            hostname="mock_host",
            reachability="unreachable",
            local_system_ip="mock_ip",
            status="normal",
            memUsage=1.0,
            connected_vManages=["192.168.0.1"],
            model="vedge-cloud",
        )

    @patch("catalystwan.api.speedtest_api.sleep")
    @patch("catalystwan.session.ManagerSession")
    def test_perform(self, mock_session, mock_sleep):
        # Arrange
        mock_session.post.return_value.json.return_value = {
            "sessionId": "id",
            "data": [{"up_speed": "up_speed", "down_speed": "down_speed"}],
        }
        mock_session.get_json.return_value = {"status": "status"}
        speed_test_api = SpeedtestAPI(mock_session)
        speed_test_api.speedtest_output = Speedtest("ip", "device_name", "ip", "device_name", None, None, None)
        speed_test_api_compare = SpeedtestAPI(mock_session)
        speed_test_api_compare.speedtest_output = Speedtest(
            "ip", "device_name", "ip", "device_name", "status", "up_speed", "down_speed"
        )
        # Act
        speed_test_api._SpeedtestAPI__perform(self.device, self.device, "blue", "red", 1)
        # Assert
        self.assertEqual(speed_test_api.speedtest_output, speed_test_api_compare.speedtest_output)

    @patch("catalystwan.session.ManagerSession")
    def test_perform_handle_error(self, mock_session):
        # Arrange
        mock_session.post.return_value.json.return_value = {"sessionId": "id", "data": []}
        mock_session.get_json.return_value = {"status": "status"}
        speed_test_api = SpeedtestAPI(mock_session)
        speed_test_api.speedtest_output = Speedtest("ip", "device_name", "ip", "device_name", None, None, None)
        speed_test_api_compare = SpeedtestAPI(mock_session)
        speed_test_api_compare.speedtest_output = Speedtest(
            "ip", "device_name", "ip", "device_name", "No speed received", None, None
        )
        # Act
        speed_test_api._SpeedtestAPI__perform(self.device, self.device, "blue", "red", 1)
        # Assert
        self.assertEqual(speed_test_api.speedtest_output, speed_test_api_compare.speedtest_output)

    @patch.object(SpeedtestAPI, "_SpeedtestAPI__perform")
    @patch.object(DeviceStateAPI, "enable_data_stream")
    @patch.object(DeviceStateAPI, "get_colors")
    @patch("catalystwan.session.ManagerSession")
    def test_speedtest(self, mock_session, mock_get_colors, mock_enable, mock_perform):
        # Arrange
        mock_enable.return_value.__enter__.return_value = None
        speed_test_api = SpeedtestAPI(mock_session)
        speed_test_api_compare = SpeedtestAPI(mock_session)
        speed_test_api_compare.speedtest_output = Speedtest(
            self.device.local_system_ip,
            self.device.hostname,
            self.device.local_system_ip,
            self.device.hostname,
            "",
            0.0,
            0.0,
        )
        # Act
        answer = speed_test_api.speedtest(self.device, self.device, 1)
        # Assert
        self.assertEqual(answer, speed_test_api_compare.speedtest_output)

    @patch.object(SpeedtestAPI, "_SpeedtestAPI__perform")
    @patch.object(DeviceStateAPI, "enable_data_stream")
    @patch.object(DeviceStateAPI, "get_colors")
    @patch("catalystwan.session.ManagerSession")
    def test_speedtest_not_reachable(self, mock_session, mock_get_colors, mock_enable, mock_perform):
        # Arrange
        mock_enable.return_value.__enter__.return_value = None
        speed_test_api = SpeedtestAPI(mock_session)
        speed_test_api_compare = SpeedtestAPI(mock_session)
        speed_test_api_compare.speedtest_output = Speedtest(
            self.device.local_system_ip,
            self.device.hostname,
            self.device.local_system_ip,
            self.device.hostname,
            "Source is unreachable and destination device is unreachable",
            0.0,
            0.0,
        )
        # Act
        answer = speed_test_api.speedtest(self.device_unreachable, self.device_unreachable, 1)
        # Assert
        self.assertEqual(answer, speed_test_api_compare.speedtest_output)

    @patch.object(SpeedtestAPI, "_SpeedtestAPI__perform")
    @patch.object(DeviceStateAPI, "enable_data_stream")
    @patch.object(DeviceStateAPI, "get_colors")
    @patch("catalystwan.session.ManagerSession")
    def test_speedtest_handle_error(self, mock_session, mock_get_colors, mock_enable, mock_perform):
        # Arrange
        mock_enable.return_value.__enter__.return_value = None
        mock_perform.side_effect = HTTPError("url", 400, "error_400", "msg", 1)
        speed_test_api = SpeedtestAPI(mock_session)
        speed_test_api_compare = SpeedtestAPI(mock_session)
        speed_test_api_compare.speedtest_output = Speedtest(
            self.device.local_system_ip,
            self.device.hostname,
            self.device.local_system_ip,
            self.device.hostname,
            "HTTP Error 400: error_400",
            0.0,
            0.0,
        )
        # Act
        answer = speed_test_api.speedtest(self.device, self.device, 1)
        # Assert
        self.assertEqual(answer, speed_test_api_compare.speedtest_output)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

"""
Module for apidocs/#/Tenant Backup Restore
"""
from __future__ import annotations

import logging
import re
from pathlib import Path
from typing import TYPE_CHECKING, List, Optional

from catalystwan.api.task_status_api import SubTaskData, Task

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession

logger = logging.getLogger(__name__)


class TenantBackupRestoreAPI:
    """
    Class for tenant backup and restore

    Scope:
        Provider-as-tenant or tenant

    Attributes:
        session (ManagerSession): logged in API client session

    Example usage:
        from catalystwan.api.tenant_backup_restore_api import TenantBackupRestoreAPI
        from catalystwan.session import create_manager_session


        tenant_session = create_manager_session(
             tenant_domain, tenantadmin, password, port)
        provider_tenant_session = create_manager_session(
            domain, admin, password, port, subdomain=tenant_domain)

        tenant_backup_restore = TenantBackupRestoreAPI(tenant_session)
        provider_backup_restore = TenantBackupRestoreAPI(provider_tenant_session)

        file_name = tenant_backup_restore.export()
        file_list = provider_backup_restore.list()
        file = tenant_backup_restore.download(file_name)
        status = provider_backup_restore.import_file(file)
        deleted_list = tenant_backup_restore.delete(file_name)
        deleted_list = provider_backup_restore.delete_all()
    """

    def __init__(self, session: ManagerSession) -> None:
        self.session = session

    def list(self) -> list:
        """Return a list of backup files stored on vManage

        Returns:
            list of filenames

        Example usage:
            fileList = ProviderBackupRestore.list()
        """
        return self.session.get_json("/dataservice/tenantbackup/list")["backup_files"]

    def export(self, timeout: int = 300) -> str:
        """Export tenant backup file from DB to vManage storage

        Args:
            timeout: Max polling time for task (default: 300 seconds)

        Returns:
            str: filename of exported tenant backup file stored on vManage

        Example usage:
            fileName = ProviderBackupRestore.export()
        """
        response = self.session.get_json("/dataservice/tenantbackup/export")
        status = Task(self.session, response["processId"]).wait_for_completed(timeout_seconds=timeout)
        string = re.search("""file location: (.*)""", status.sub_tasks_data[0].activity[-1])
        assert string, "File location not found."
        return string.group(1)

    def delete(self, file: str) -> List[str]:
        """Delete tenant backup file

        Args:
            file (str): device ID (usually system-ip)

        Returns:
            http response for delete operation

        Example usage:
            message = ProviderBackupRestore.delete(fileName)
        """
        url = f"/dataservice/tenantbackup/delete?fileName={file}"
        return self.session.delete(url).json()["Deleted"]

    def delete_all(self) -> List[str]:
        """Delete all tenant backup files

        Returns:
            http response for delete operation

        Example usage:
            message = ProviderBackupRestore.delete_all()
        """
        return self.delete("all")

    def download(self, file: str, download_dir: Optional[Path] = None) -> Path:
        """Download tenant backup file

        Args:
            file: full path or base name of tenant backup file
            download_dir: download directory (defaul: current working directory)

        Returns:
            Path to downloaded tenant backup file

        Example usage:
            file = ProviderBackupRestore.download(fileName)
        """

        if "/dataservice/tenantbackup/download/" not in file:
            tenant_id = file.split("_")[1]
            file = f"/dataservice/tenantbackup/download/{tenant_id}/{file}"

        download_dir = download_dir if download_dir else Path.cwd()
        download = download_dir / Path(file).name
        self.session.get_file(file, download)
        return download

    def import_file(self, file: Path, timeout: int = 300) -> SubTaskData:
        """Upload the file for tenant import to the DB and poll for Success

        Args:
            file: The path of the file to be upladed
            timeout: Max polling time for task (default: 300 seconds)

        Returns:
            TaskStatus object with Success and Activity messages

        Example usage:
             status = ProviderBackupRestore.import_file(file)
        """

        url = "/dataservice/tenantbackup/import"
        files = {"file": (file.name, open(str(file), "rb"))}
        response = self.session.post(url, data={}, files=files)
        return (
            Task(self.session, response.json()["processId"])
            .wait_for_completed(timeout_seconds=timeout)
            .sub_tasks_data[0]
        )



# Copyright 2022 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

import logging
from http import HTTPStatus
from typing import TYPE_CHECKING, List, Union, cast, overload

from requests import Response
from typing_extensions import deprecated

from catalystwan.dataclasses import (
    Certificate,
    CloudConnectorData,
    CloudServicesSettings,
    Organization,
    Password,
    ServiceConfigurationData,
    SoftwareInstallTimeout,
    Vbond,
)
from catalystwan.endpoints.administration_user_and_group import (
    ActiveSession,
    AdministrationUserAndGroup,
    InvalidateSessionMessage,
    ResourceGroup,
    ResourceGroupSwitchRequest,
    ResourceGroupUpdateRequest,
    SessionsDeleteRequest,
    User,
    UserGroup,
    UserResetRequest,
    UserRole,
    UserUpdateRequest,
)
from catalystwan.exceptions import CatalystwanDeprecationWarning, CatalystwanException
from catalystwan.typed_list import DataSequence
from catalystwan.utils.creation_tools import asdict, create_dataclass

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession

logger = logging.getLogger(__name__)


class UsersAPI:
    """Class implementing methods for user managment.

    Attributes:
        session: logged in API client session

    Usage example:
        # Create session
        session = create_manager_session(...)
        # Get information about all users
        all_users = session.api.users.get()
    """

    def __init__(self, session: ManagerSession) -> None:
        self.session = session
        self._endpoints = AdministrationUserAndGroup(session)

    def get(self) -> DataSequence[User]:
        """List all users

        Returns:
            DataSequence[User]: List-like object representing users
        """
        return self._endpoints.find_users()

    def get_role(self) -> UserRole:
        """Get currently logged user role

        Returns:
            UserRole: Currently logged user role information
        """
        return self._endpoints.find_user_role()

    def get_auth_type(self) -> str:
        """Get currently logged user authentication type

        Returns:
            str: Currently logged user authentication type
        """
        return self._endpoints.find_user_auth_type().user_auth_type

    def create(self, user: User):
        """Creates a new user

        Args:
            user (User): Definition of new user to be created
        """
        self._endpoints.create_user(user)

    def update(self, user_update_request: UserUpdateRequest):
        """Updates existing user

        Args:
            user_update_request (UserUpdateRequest): User attributes to be updated
        """
        self._endpoints.update_user(user_update_request.username, user_update_request)

    def update_password(self, username: str, new_password: str):
        """Updates exisiting user password

        Args:
            username (str): Name of the user
            new_password (str): New password for given user
        """
        update_password_request = UserUpdateRequest(
            username=username, password=new_password, current_user_password=self.session.password
        )  # type: ignore
        self._endpoints.update_password(username, update_password_request)

    def reset(self, username: str):
        """Resets given user (unlocks blocked user eg. after number of unsuccessfull login attempts)

        Args:
            username (str): Name of the user to be unlocked
        """
        self._endpoints.reset_user(UserResetRequest(username=username))

    def delete(self, username: str):
        """Deletes given user

        Args:
            username (str): Name of the user to be deleted
        """
        self._endpoints.delete_user(username)


class UserGroupsAPI:
    """Class implementing methods for user group management."""

    def __init__(self, session: ManagerSession):
        self.session = session
        self._endpoints = AdministrationUserAndGroup(session)

    def get(self) -> DataSequence[UserGroup]:
        """List all user groups

        Returns:
            DataSequence[UserGroup]: List-like object representing user groups
        """
        return self._endpoints.find_user_groups()

    def create(self, user_group: UserGroup):
        """Creates a new user group

        Args:
            user_group (UserGroup): Definition of user group to be created
        """
        self._endpoints.create_user_group(user_group)

    def update(self, user_group: UserGroup):
        """Updates existing user group

        Args:
            user_group (UserGroup): User group attributes to be updated
        """
        self._endpoints.update_user_group(user_group.group_name, user_group)

    def delete(self, group_name: str):
        """Deletes given user group

        Args:
            group_name (str): Name of the user group to be deleted
        """
        self._endpoints.delete_user_group(group_name)


class ResourceGroupsAPI:
    """Class implementing methods for resource groups management."""

    def __init__(self, session: ManagerSession):
        self.session = session
        self._endpoints = AdministrationUserAndGroup(session)

    def get(self) -> DataSequence[ResourceGroup]:
        """List all resource groups

        Returns:
            DataSequence[ResourceGroup]: List-like object containing user groups information
        """
        return self._endpoints.find_resource_groups()

    def create(self, resource_group: ResourceGroup):
        """Creates a new resource group

        Args:
            resource_group (ResourceGroup): Definition of new resource group to be created
        """
        self._endpoints.create_resource_group(resource_group)

    def update(self, resource_group_update_request: ResourceGroupUpdateRequest):
        """Updates existing resource group

        Args:
            resource_group_update_request (ResourceGroupUpdateRequest): Object containing existing
            resource group id and attributes to be updated
        """
        self._endpoints.update_resource_group(resource_group_update_request.id, resource_group_update_request)

    def switch(self, resource_group_name: str):
        """Switch to view only a specific resource group (for global admin only)

        Args:
            resource_group_name (str): Name of resource group to switch view
        """
        switch_request = ResourceGroupSwitchRequest(resource_group_name=resource_group_name)
        self._endpoints.switch_resource_group(switch_request)

    def delete(self, resource_group_id: str):
        """Deletes a given resource group

        Args:
            resource_group_id (str): Resource group id
        """
        self._endpoints.delete_resource_group(resource_group_id)


class SessionsAPI:
    """Class implementing methods for vmanage sessions management."""

    def __init__(self, session: ManagerSession):
        self.session = session
        self._endpoints = AdministrationUserAndGroup(session)

    def get(self) -> DataSequence[ActiveSession]:
        """List all active sessions

        Returns:
            DataSequence[ActiveSession]: List-like object representing active user sessions
        """
        return self._endpoints.get_active_sessions()

    def invalidate(self, sessions: List[ActiveSession]) -> InvalidateSessionMessage:
        """Invalidates given sessions

        Args:
            sessions (List[ActiveSession]): List of active sessions

        Returns:
            InvalidateSessionMessage: Information about invalidation result
        """
        sessions_delete_request = SessionsDeleteRequest.from_active_session_list(sessions)
        return self._endpoints.remove_sessions(sessions_delete_request)


class ClusterManagementAPI:
    """Covers clusterManagement API calls.

    Attributes:
        session: logged in API admin session

    Example usage:
        # Create session
        session = create_manager_session(...)
        # Get health status
        health_status = session.api.cluster_management.get_cluster_management_health_status()
    """

    def __init__(self, session: ManagerSession) -> None:
        self.session = session

    def modify_cluster_setup(self, service_configuration: ServiceConfigurationData) -> bool:
        """Updates vManage cluster configuration.

        Args:
            service_configuration: vManage cluster config
        """
        url_path = "/dataservice/clusterManagement/setup"
        data = asdict(service_configuration)  # type: ignore
        response = self.session.put(url_path, json=data)
        return True if response.status_code == 200 else False

    def get_cluster_management_health_status(self) -> Union[dict, list]:
        """Gets Cluster Management Service Reachability health status.

        Returns:
            TODO return health status dataclass
        """
        url_path = "/dataservice/clusterManagement/health/status"
        return self.session.get_data(url_path)


class AdministrationSettingsAPI:
    def __init__(self, session: ManagerSession) -> None:
        """Covers Administration Settings API calls.

        Args:
            session: logged in API admin session

        Example usage:
            - to update e.g. password:
                password = Password(old_password="qwer1234", new_password="asdf9876")
                AdministrationSettingsAPI(session).update(password)
        """
        self.session = session

    def get_sdavc_cloud_connector_config(self) -> CloudConnectorData:
        """Gets SD-AVC Cloud Connector Config.

        Returns:
            CloudConnector dataclass config
        """
        url_path = "/dataservice/sdavc/cloudconnector"
        response = cast(dict, self.session.get_json(url_path))
        return create_dataclass(CloudConnectorData, response)

    def enable_sdavc_cloud_connector(self, cloud_connector: CloudConnectorData) -> bool:
        """Enables SD-AVC Cloud Connector on vManage."""
        url_path = "/dataservice/sdavc/cloudconnector"
        data = asdict(cloud_connector)  # type: ignore
        response = self.session.post(url_path, json=data)
        return True if response.status_code == 200 else False

    def disable_sdavc_cloud_connector(self) -> bool:
        """Disables SD-AVC Cloud Connector on vManage."""
        url_path = "/dataservice/sdavc/cloudconnector"
        data = {"cloudEnabled": False}
        response = self.session.put(url_path, data)
        return True if response.status_code == 200 else False

    def get_cloud_services(self) -> CloudServicesSettings:
        url_path = "/dataservice/settings/configuration/cloudservices"
        data = self.session.get_data(url_path)[0]
        return create_dataclass(CloudServicesSettings, data)

    def set_cloud_services(self, config: CloudServicesSettings) -> bool:
        url_path = "/dataservice/settings/configuration/cloudservices"
        response = self.session.post(url_path, asdict(config))  # type: ignore
        return True if response.status_code == 200 else False

    def get_cloud_on_ramp_for_saas_mode(self):
        """Get information about Cloud on Ramp for Saas mode"""
        url_path = "/dataservice/settings/configuration/cloudx"
        return self.session.get_data(url_path)

    def enable_cloud_on_ramp_for_saas_mode(self, disable=False):
        """Enable or disable COR for SaaS"""
        url_path = "/dataservice/settings/configuration/cloudx"
        data = {"mode": "on"} if not disable else {"mode": "off"}
        response = self.session.put(url_path, data)
        return True if response.status_code == 200 else False

    @deprecated(
        "Use .endpoints.configuration_settings.get_organizations() instead", category=CatalystwanDeprecationWarning
    )
    def get_organization(self) -> Organization:
        endpoint = "/dataservice/settings/configuration/organization"

        return create_dataclass(Organization, self.session.get_data(endpoint)[0])

    def get_software_install_timeout(self) -> SoftwareInstallTimeout:
        url_path = "/settings/configuration/softwareMaintenance"

        return create_dataclass(SoftwareInstallTimeout, self.session.get_data(url_path)[0])

    def set_software_install_timeout(self, download_timeout_min: int, activate_timeout_min: int):
        MINIMAL_DOWNLOAD_TIMEOUT_MIN = 60
        MINIMAL_ACTIVATE_TIMEOUT_MIN = 30
        assert download_timeout_min >= MINIMAL_DOWNLOAD_TIMEOUT_MIN
        assert activate_timeout_min >= MINIMAL_ACTIVATE_TIMEOUT_MIN

        url_path = "/settings/configuration/softwareMaintenance"
        data = asdict(SoftwareInstallTimeout(download_timeout_min, activate_timeout_min))  # type: ignore
        response = self.session.post(url_path, json=data)
        return True if response.status_code == 200 else False

    @overload
    def update(self, payload: Password) -> bool:
        ...

    @overload
    def update(self, payload: Certificate) -> bool:
        ...

    @overload
    def update(self, payload: Vbond) -> bool:
        ...

    @overload
    def update(self, payload: Organization) -> bool:
        ...

    def update(self, payload: Union[Organization, Certificate, Password, Vbond]) -> bool:
        json_payload = asdict(payload)  # type: ignore
        if isinstance(payload, Organization):
            response = self.__update_organization(json_payload)
        elif isinstance(payload, Certificate):
            response = self.__update_certificate(payload)
        elif isinstance(payload, Password):
            response = self.__update_password(json_payload)
        elif isinstance(payload, Vbond):
            response = self.__update_vbond(json_payload)
        else:
            raise CatalystwanException(f"Not supported payload type: {type(payload).__name__}")

        return True if response.status_code == HTTPStatus.OK else False

    def __update_password(self, payload: dict) -> Response:
        endpoint = "/dataservice/admin/user/profile/password"
        response = self.session.put(endpoint, json=payload)
        logger.info("Password changed.")
        return response

    def __update_certificate(self, payload: Certificate) -> Response:
        json_payload = asdict(payload)  # type: ignore
        endpoint = "/dataservice/settings/configuration/certificate"
        return self.session.put(endpoint, json=json_payload)

    def __update_vbond(self, payload: dict) -> Response:
        endpoint = "/dataservice/settings/configuration/device"
        return self.session.post(endpoint, json=payload)

    @deprecated(
        "Use .endpoints.configuration_settings.edit_organizations() instead", category=CatalystwanDeprecationWarning
    )
    def __update_organization(self, payload: dict) -> Response:
        endpoint = "/dataservice/settings/configuration/organization"
        del payload["controlConnectionUp"]
        return self.session.put(endpoint, json=payload)



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

from typing import TYPE_CHECKING

from catalystwan.typed_list import DataSequence
from catalystwan.utils.creation_tools import create_dataclass
from catalystwan.utils.dashboard import (
    CertificatesStatus,
    Count,
    DeviceHealth,
    DeviceHealthOverview,
    DevicesHealth,
    LicensedDevices,
    TenantStatus,
    TransportHealth,
    TunnelHealth,
)

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession


class DashboardAPI:
    """Dashboard API gathers information from vManage dashboard.

    Attributes:
        session (ManagerSession): logged in API client session

    Usage example:
        # Create session
        session = create_manager_session(...)
        # Get information about tenant status
        tenant_status = session.api.dashboard.get_tenant_status()
    """

    def __init__(self, session: ManagerSession):
        self.session = session

    def get_vmanages_count(self) -> DataSequence[Count]:
        """
        Get information about number of vmanages.

        Returns:
            DataSequance of Count dataclass with vManages
        """
        vmanages = self.session.get_data("/dataservice/clusterManagement/health/summary")

        vmanages = DataSequence(Count, [create_dataclass(Count, vmanage) for vmanage in vmanages])
        for vmanage in vmanages:
            vmanage.status_list = DataSequence(Count, [create_dataclass(Count, item) for item in vmanage.status_list])

        return vmanages

    def get_devices_count(self) -> DataSequence[Count]:
        """
        Get information about number of devices (does not include vManages).

        Returns:
            DataSequance of Count dataclass with devices
        """
        devices = self.session.get_data("/dataservice/network/connectionssummary")

        devices = DataSequence(Count, [create_dataclass(Count, device) for device in devices])
        for device in devices:
            device.status_list = DataSequence(Count, [create_dataclass(Count, item) for item in device.status_list])

        return devices

    def get_certificates_status(self) -> DataSequence[CertificatesStatus]:
        """
        Get information about status of certificates.

        Returns:
            DataSequance of CertificatesStatus dataclass with devices
        """
        devices = self.session.get_data("/dataservice/certificate/stats/summary")

        return DataSequence(CertificatesStatus, [create_dataclass(CertificatesStatus, device) for device in devices])

    def get_control_statuses_count(self) -> DataSequence[Count]:
        """
        Get information about control statuses.

        Returns:
            DataSequance of Count dataclass with control statuses
        """
        statuses = self.session.get_data("/dataservice/device/control/count")

        statuses = DataSequence(Count, [create_dataclass(Count, status) for status in statuses])
        for status in statuses:
            status.status_list = DataSequence(Count, [create_dataclass(Count, item) for item in status.status_list])

        return statuses

    def get_bfd_connectivity_count(self) -> DataSequence[Count]:
        """
        Get information about bfd connectivity count.

        Returns:
            DataSequance of Count dataclass with bfd connectivity
        """
        response = self.session.get("/dataservice/device/bfd/sites/summary")

        bfd_response = response.dataseq(Count)
        for item in bfd_response:
            item.status_list = DataSequence(
                Count, [create_dataclass(Count, bfd_connection) for bfd_connection in item.status_list]  # type:ignore
            )

        return bfd_response

    def get_edges_inventory_count(self) -> DataSequence[Count]:
        """
        Get information about edges inventory.

        Returns:
            DataSequance of Count dataclass with edges inventory count
        """
        response = self.session.get("/dataservice/device/vedgeinventory/summary")

        edges_inventory = response.dataseq(Count)

        return edges_inventory

    def get_transport_interface_distribution(self) -> DataSequence[Count]:
        """
        Get information about transport interface distribution.

        Returns:
            DataSequance of Count dataclass with transport interface distribution
        """
        response = self.session.get("/dataservice/device/tlocutil")

        transport_interface = response.dataseq(Count)

        return transport_interface

    def get_tenant_status(self) -> DataSequence[TenantStatus]:
        """
        Get information about tenant status, including:
        - control status
        - site health
        - vEdge health
        - vSmart status

        Returns:
            DataSequance of TenantStatus dataclass with tenant status information
        """
        response = self.session.get("/dataservice/tenantstatus")

        transport_interface = response.dataseq(TenantStatus)

        return transport_interface

    def get_devices_health(self) -> DevicesHealth:
        """
        Get information about devices health

        Returns:
            DevicesHealth object
        """
        response = self.session.get_json("/dataservice/health/devices")
        devices_health = create_dataclass(DevicesHealth, response)

        devices_health.devices = DataSequence(
            DeviceHealth, [create_dataclass(DeviceHealth, device) for device in devices_health.devices]  # type: ignore
        )

        return devices_health

    def get_licensed_devices(self) -> DataSequence[LicensedDevices]:
        """
        Get information about licensed devices.

        Returns:
            DataSequance of LicensedDevices dataclass with licences information.
        """
        response = self.session.get("/dataservice/msla/monitoring/licensedDeviceCount")

        return response.dataseq(LicensedDevices)

    def get_devices_health_overview(self) -> DataSequence[DeviceHealthOverview]:
        """
        Get information about health overview devices.

        Returns:
            DataSequance of DeviceHealthOverview dataclass with health information.
        """
        response = self.session.get_json("/dataservice/health/devices/overview")

        return DataSequence(DeviceHealthOverview, [create_dataclass(DeviceHealthOverview, response)])

    def get_transport_health(self) -> DataSequence[TransportHealth]:
        """
        Get information about loss percentage, latency and jitter for all links and combinations of colors.

        Returns:
            DataSequance of TransportHealth dataclass with loss percentage, latency and jitter information.
        """
        response = self.session.get("/dataservice/statistics/approute/transport/summary/loss_percentage")

        return response.dataseq(TransportHealth)

    def get_tunnel_health(self) -> DataSequence[TunnelHealth]:
        """
        Get information about state, loss percentage, latency and jitter for tunnels.

        Returns:
            DataSequance of TunnelHealth dataclass with tunnel health information.
        """
        response = self.session.get("/dataservice/statistics/approute/tunnels/health/latency")

        return response.dataseq(TunnelHealth)



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from typing import List, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, ConfigDict, Field

from catalystwan.api.configuration_groups.parcel import Default, Global, Variable

EndpointProtocol = Literal[
    "tcp",
    "udp",
]

TrackerType = Literal["endpoint"]

EndpointTrackerType = Literal["static-route"]

CombineBoolean = Literal[
    "and",
    "or",
]


class EndpointTcpUdp(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    protocol: Optional[Union[Variable, Global[EndpointProtocol]]] = None
    port: Optional[Union[Variable, Global[int]]] = None


class TrackerData(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: Union[Variable, Global[str]] = Field(serialization_alias="trackerName", validation_alias="trackerName")
    endpoint_api_url: Optional[Union[Variable, Global[str]]] = Field(
        serialization_alias="endpointApiUrl", validation_alias="endpointApiUrl", default=None
    )
    endpoint_dns_name: Optional[Union[Variable, Global[str]]] = Field(
        serialization_alias="endpointDnsName", validation_alias="endpointDnsName", default=None
    )
    endpoint_ip: Optional[Union[Variable, Global[str]]] = Field(
        serialization_alias="endpointIp", validation_alias="endpointIp", default=None
    )
    endpoint_tcp_udp: Optional[EndpointTcpUdp] = Field(
        serialization_alias="endpointTcpUdp", validation_alias="endpointTcpUdp", default=None
    )
    interval: Optional[Union[Global[int], Variable, Default[int]]] = Default[int](value=60)
    multiplier: Optional[Union[Global[int], Variable, Default[int]]] = Default[int](value=3)
    threshold: Optional[Union[Global[int], Variable, Default[int]]] = Default[int](value=300)
    endpoint_tracker_type: Optional[Union[Global[EndpointTrackerType], Variable, Default[EndpointTrackerType]]] = Field(
        serialization_alias="endpointTrackerType",
        validation_alias="endpointTrackerType",
        default=Default[EndpointTrackerType](value="static-route"),
    )
    tracker_type: Optional[Union[Global[TrackerType], Variable, Default[TrackerType]]] = Field(
        serialization_alias="trackerType",
        validation_alias="trackerType",
        default=Default[TrackerType](value="endpoint"),
    )


class TrackerCreationPayload(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: str
    description: Optional[str] = None
    data: TrackerData
    metadata: Optional[dict] = None


class TrackerRef(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    tracker_ref: Global[UUID] = Field(serialization_alias="trackerRef", validation_alias="trackerRef")


class TrackerGroupData(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    tracker_refs: List[TrackerRef] = Field(serialization_alias="trackerRefs", validation_alias="trackerRefs")
    combine_boolean: Union[Global[CombineBoolean], Variable, Default[CombineBoolean]] = Field(
        serialization_alias="combineBoolean",
        validation_alias="combineBoolean",
        default=Default[CombineBoolean](value="or"),
    )


class TrackerGroupCreationPayload(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    name: str
    description: Optional[str] = None
    data: TrackerGroupData
    metadata: Optional[dict] = None


class TrackerAssociationPayload(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True, populate_by_name=True)

    parcel_id: str = Field(serialization_alias="parcelId", validation_alias="parcelId")



# Copyright 2024 Cisco Systems, Inc. and its affiliates

from ipaddress import IPv4Address, IPv6Address
from typing import List, Optional, Union

from pydantic import AliasPath, BaseModel, ConfigDict, Field

from catalystwan.api.configuration_groups.parcel import Default, Global, Variable, _ParcelBase, as_default, as_global


class PubkeyChainItem(BaseModel):
    model_config = ConfigDict(
        extra="forbid",
    )
    key_string: Global[str] = Field(
        validation_alias="keyString",
        serialization_alias="keyString",
        pattern="^AAAA[0-9A-Za-z+/]+[=]{0,3}$",
        description="Set the RSA key string",
    )

    # Literal["ssh-rsa"]
    key_type: Global[str] = Field(
        default=Global[str](value="ssh-rsa"),
        serialization_alias="keyType",
        validation_alias="keyType",
        description="Only RSA is supported",
    )


class UserItem(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)

    name: Union[Global[str], Variable] = Field(description="Set the username")
    password: Union[Global[str], Variable] = Field(
        description=(
            "Set the user password [Note: Catalyst SD-WAN Manager will encrypt this field before saving."
            "Cleartext strings will not be returned back to the user in GET responses for sensitive fields.]"
        )
    )
    privilege: Union[Global[str], Variable, Default[str], None] = Field(
        None, description="Set Privilege Level for this user"
    )
    pubkey_chain: Optional[List[PubkeyChainItem]] = Field(
        default=None,
        validation_alias="pubkeyChain",
        serialization_alias="pubkeyChain",
        description="List of RSA public-keys per user",
        max_length=2,
    )

    def add_pubkey_chain_item(self, key: str) -> PubkeyChainItem:
        item = PubkeyChainItem(key_string=as_global(key))

        if self.pubkey_chain:
            self.pubkey_chain.append(item)
        else:
            self.pubkey_chain = [item]

        return item


class RadiusServerItem(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)

    address: Union[Global[IPv4Address], Global[IPv6Address]] = Field(description="Set IP address of Radius server")
    auth_port: Union[Global[int], Default[int], Variable, None] = Field(
        default=as_default(1812),
        validation_alias="authPort",
        serialization_alias="authPort",
        description="Set Authentication port to use to connect to Radius server",
    )
    acct_port: Union[Global[int], Default[int], Variable, None] = Field(
        default=as_default(1813),
        validation_alias="acctPort",
        serialization_alias="acctPort",
        description="Set Accounting port to use to connect to Radius server",
    )

    timeout: Union[Variable, Global[int], Default[int], None] = Field(
        default=as_default(5),
        description="Configure how long to wait for replies from the Radius server",
    )
    key: Global[str] = Field(
        description=(
            "Set the Radius server shared key [Note: Catalyst SD-WAN Manager will encrypt "
            "this field before saving. Cleartext strings will not be returned back "
            "to the user in GET responses for sensitive fields.]"
        )
    )
    secret_key: Union[Global[str], Variable, None] = Field(
        default=None,
        validation_alias="secretKey",
        serialization_alias="secretKey",
        description="Set the TACACS server shared type 7 encrypted key",
    )
    # Literal["6", "7"]
    key_enum: Union[Global[str], Default[None], None] = Field(
        default=None,
        validation_alias="keyEnum",
        serialization_alias="keyEnum",
        description="Type of encyption. To be used for type 6",
    )
    # Literal["key", "pac"]
    key_type: Union[Global[str], Default[str], Variable, None] = Field(
        default=as_default("key"), validation_alias="keyType", serialization_alias="keyType", description="key type"
    )

    retransmit: Union[Variable, Global[int], Default[int], None] = Field(
        default=as_default(3), description="Configure how many times to contact this Radius server"
    )


class Radius(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)
    group_name: Global[str] = Field(
        validation_alias="groupName", serialization_alias="groupName", description="Set Radius server Group Name"
    )

    vpn: Union[Global[int], Default[int], None] = Field(
        default=None, description="Set VPN in which Radius server is located"
    )
    source_interface: Union[Global[str], Default[None], Variable, None] = Field(
        default=None,
        validation_alias="sourceInterface",
        serialization_alias="sourceInterface",
        description="Set interface to use to reach Radius server",
    )
    server: List[RadiusServerItem] = Field(description="Configure the Radius server")

    @staticmethod
    def generate_radius_server(
        address: Union[IPv4Address, IPv6Address],
        key: str,
        auth_port: Optional[int] = None,
        acct_port: Optional[int] = None,
        timeout: Optional[int] = None,
        secret_key: Optional[str] = None,
        key_enum: Optional[str] = None,
        key_type: Optional[str] = None,
        retransmit: Optional[int] = None,
    ) -> RadiusServerItem:
        item = RadiusServerItem(
            address=as_global(address),
            auth_port=as_global(auth_port),
            acct_port=as_global(acct_port),
            key=as_global(key),
            retransmit=as_global(retransmit),
            timeout=as_global(timeout),
            secret_key=as_global(secret_key),
            key_enum=as_global(key_enum),
            key_type=as_global(key_type),
        )

        return item


class TacacsServerItem(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)

    address: Union[Global[IPv4Address], Global[IPv6Address]] = Field(description="Set IP address of TACACS server")
    port: Union[Variable, Global[int], Default[int], None] = Field(default=None, description="TACACS Port")
    timeout: Union[Variable, Global[int], Default[int], None] = Field(
        default=None,
        description="Configure how long to wait for replies from the TACACS server",
    )
    key: Global[str] = Field(
        description=(
            "Set the TACACS server shared key [Note: Catalyst SD-WAN Manager will encrypt"
            "this field before saving. Cleartext strings will not be returned back"
            "to the user in GET responses for sensitive fields.]"
        )
    )
    secret_key: Union[Global[str], Variable, None] = Field(
        default=None,
        validation_alias="secretKey",
        serialization_alias="secretKey",
        description="Set the TACACS server shared type 7 encrypted key",
    )
    # Literal["6", "7"]
    key_enum: Union[Global[str], Default[None], None] = Field(
        default=None,
        validation_alias="keyEnum",
        serialization_alias="keyEnum",
        description="Type of encyption. To be used for type 6",
    )


class Tacacs(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)

    group_name: Global[str] = Field(
        validation_alias="groupName", serialization_alias="groupName", description="Set TACACS server Group Name"
    )
    vpn: Union[Global[int], Default[int], None] = Field(
        default=None, description="Set VPN in which TACACS server is located"
    )
    source_interface: Union[Global[str], Default[str], None] = Field(
        default=None,
        validation_alias="sourceInterface",
        serialization_alias="sourceInterface",
        description="Set interface to use to reach TACACS server",
    )
    server: List[TacacsServerItem] = Field(description="Configure the TACACS server")

    @staticmethod
    def generate_tacacs_server(
        address: Union[IPv4Address, IPv6Address],
        key: str,
        port: Optional[int] = None,
        timeout: Optional[int] = None,
        secret_key: Optional[str] = None,
        key_enum: Optional[str] = None,
    ) -> TacacsServerItem:
        item = TacacsServerItem(
            address=as_global(address),
            key=as_global(key),
            port=as_global(port),
            timeout=as_global(timeout),
            secret_key=as_global(secret_key),
            key_enum=as_global(key_enum),
        )

        return item


class AccountingRuleItem(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)
    rule_id: Global[str] = Field(
        validation_alias="ruleId", serialization_alias="ruleId", description="Configure Accounting Rule ID"
    )
    # Literal['commands', 'exec', 'network', 'system']
    method: Global[str] = Field(description="Configure Accounting Method")
    # Literal['1', '15']
    level: Union[Global[str], Default[None], None] = Field(None, description="Privilege level when method is commands")
    start_stop: Union[Variable, Global[bool], Default[bool], None] = Field(
        default=None,
        validation_alias="startStop",
        serialization_alias="startStop",
        description="Record start and stop without waiting",
    )
    group: Global[List[str]] = Field(description="Use Server-group")


class AuthorizationRuleItem(BaseModel):
    model_config = ConfigDict(extra="forbid", populate_by_name=True)
    rule_id: Global[str] = Field(
        validation_alias="ruleId", serialization_alias="ruleId", description="Configure Authorization Rule ID"
    )
    # Literal["commands"]
    method: Global[str]
    # Literal['1', '15']
    level: Global[str] = Field(description="Privilege level when method is commands")
    group: Global[List[str]] = Field(description="Use Server-group")
    if_authenticated: Union[Global[bool], Default[bool], None] = Field(
        default=None,
        validation_alias="ifAuthenticated",
        serialization_alias="ifAuthenticated",
        description="Succeed if user has authenticated",
    )


class AAA(_ParcelBase):
    authentication_group: Union[Variable, Global[bool], Default[bool]] = Field(
        default=as_default(False),
        validation_alias=AliasPath("data", "authenticationGroup"),
        description="Authentication configurations parameters",
    )
    accounting_group: Union[Variable, Global[bool], Default[bool]] = Field(
        default=as_default(False),
        validation_alias=AliasPath("data", "accountingGroup"),
        description="Accounting configurations parameters",
    )
    # local, radius, tacacs
    server_auth_order: Global[List[str]] = Field(
        validation_alias=AliasPath("data", "serverAuthOrder"),
        min_length=1,
        max_length=4,
        description="ServerGroups priority order",
    )
    user: Optional[List[UserItem]] = Field(default=None, description="Create local login account", min_length=1)
    radius: Optional[List[Radius]] = Field(
        default=None, validation_alias=AliasPath("data", "radius"), description="Configure the Radius serverGroup"
    )
    tacacs: Optional[List[Tacacs]] = Field(
        default=None, validation_alias=AliasPath("data", "tacacs"), description="Configure the TACACS serverGroup"
    )
    accounting_rule: Optional[List[AccountingRuleItem]] = Field(
        default=None, validation_alias=AliasPath("data", "accountingRule"), description="Configure the accounting rules"
    )
    authorization_console: Union[Variable, Global[bool], Default[bool]] = Field(
        default=as_default(False),
        validation_alias=AliasPath("data", "authorizationConsole"),
        description="For enabling console authorization",
    )
    authorization_config_commands: Union[Variable, Global[bool], Default[bool]] = Field(
        default=as_default(False),
        validation_alias=AliasPath("data", "authorizationConfigCommands"),
        description="For configuration mode commands.",
    )
    authorization_rule: Optional[List[AuthorizationRuleItem]] = Field(
        default=None,
        validation_alias=AliasPath("data", "authorizationRule"),
        description="Configure the Authorization Rules",
    )

    def add_authorization_rule(
        self, rule_id: str, method: str, level: str, group: List[str], if_authenticated: bool
    ) -> AuthorizationRuleItem:
        item = AuthorizationRuleItem(
            rule_id=as_global(rule_id),
            method=as_global(method),
            level=as_global(level),
            group=Global[List[str]](value=group),
            if_authenticated=as_global(if_authenticated),
        )

        if self.authorization_rule:
            self.authorization_rule.append(item)
        else:
            self.authorization_rule = [item]

        return item

    def add_accounting_rule(
        self, rule_id: str, method: str, level: str, group: List[str], start_stop: bool
    ) -> AccountingRuleItem:
        item = AccountingRuleItem(
            rule_id=as_global(rule_id),
            method=as_global(method),
            level=as_global(level),
            start_stop=as_global(start_stop),
            group=Global[List[str]](value=group),
        )

        if self.accounting_rule:
            self.accounting_rule.append(item)
        else:
            self.accounting_rule = [item]

        return item

    def add_radius_group(
        self, group_name: str, vpn: int, radius_servers: List[RadiusServerItem], source_interface: Optional[str] = None
    ) -> Radius:
        radius = Radius(
            group_name=as_global(group_name),
            vpn=as_global(vpn),
            server=radius_servers,
            source_interface=Default[None](value=None),
        )

        if self.radius:
            self.radius.append(radius)
        else:
            self.radius = [radius]

        return radius



# Copyright 2023 Cisco Systems, Inc. and its affiliates

from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from catalystwan.dataclasses import ResourcePoolData
from catalystwan.typed_list import DataSequence

if TYPE_CHECKING:
    from catalystwan.session import ManagerSession

logger = logging.getLogger(__name__)


class ResourcePoolAPI:
    """API methods of Resource Pool.

    Attributes:
        session: logged in API client session
    Usage example:
        # Create session
        session = create_manager_session(...)
        # Create device vpn
        resource_poold_data = session.api.resource_pool.create()
    """

    URL = "/dataservice/resourcepool/resource/vpn"

    def __init__(self, session: ManagerSession):
        self.session = session

    def get(self, tenant_id: str, tenant_vpn: int) -> DataSequence[ResourcePoolData]:
        """Get device vpn for tenant.

        Args:
            tenant_id: The tenant id.
            tenant_vpn: The number of tenant vpn.

        Returns:
            DataSequence[ResourcePoolData] for tenant.

        TODO note - tenant id doesn't work, need to enter tenant organization name.
        """
        parameters = {"tenantId": tenant_id, "tenantVpn": tenant_vpn}
        response = self.session.get(url=ResourcePoolAPI.URL, params=parameters)
        return response.dataseq(ResourcePoolData)

    def create(self) -> DataSequence[ResourcePoolData]:
        """Create device vpn.

        Returns:
            DataSequence[ResourcePoolData].
        """
        response = self.session.put(url=ResourcePoolAPI.URL)
        logger.info("Create Resource poll vpn.")
        return response.dataseq(ResourcePoolData)

    def delete(self, tenant_id: str, tenant_vpn: int) -> bool:
        """Delete device vpn for tenant.

        Args:
            tenant_id: The tenant id.
            tenant_vpn: The number of tenant vpn.

        Returns:
            bool: True if all ok, False like something wrong

        TODO note - tenant id doesn't work, need to enter tenant organization name.
        """
        parameters = {"tenantId": tenant_id, "tenant_vpn": tenant_vpn}
        response = self.session.delete(url=ResourcePoolAPI.URL, params=parameters)
        return response.ok
